#!/usr/bin/env python

""" MultiQC module to parse output from MultiVCFAnalyzer """

from __future__ import print_function
from collections import OrderedDict
import logging
import json

from multiqc.plots import table
from multiqc.plots import bargraph
from collections import OrderedDict
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ MultiVCFAnalyzer module """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='MultiVCFAnalyzer', anchor='multivcfanalyzer',
        href="https://github.com/alexherbig/MultiVCFAnalyzer",
        info="""combines multiple VCF files in a coherent way, can produce summary statistics and downstream analysis formats for phylogeny reconstruction.""")

        # Find and load any MultiVCFAnalyzer reports
        self.mvcf_data = dict()

        # Find and load JSON file
        for f in self.find_log_files('multivcfanalyzer', filehandles=True):
            self.parse_data(f)

        # Filter samples
        self.mvcf_data = self.ignore_samples(self.mvcf_data)

        # Return if no samples found
        if len(self.mvcf_data) == 0:
            raise UserWarning

        # Add in extra columns to data file
        self.compute_perc_hets()

        # Add in extra stuff like Snp(hom) to data
        self.computeSnpHom()

        # Save data output file
        self.write_data_file(self.mvcf_data, 'multiqc_multivcfanalyzer')

        # Add to General Statistics
        self.addSummaryMetrics()

        # Add MultiVCFAnalyzer Table Section
        self.add_section (
        name = 'Summary metrics',
        anchor = 'mvcf_table',
        plot = self.addTable())

        # Add MultiVCFAnalyzer Barplot Section
        self.add_section (
        name = 'Call statistics barplot',
        anchor = 'mvcf-barplot',
        helptext = """
        MultiVCFAnalyzer has a filtering step during the merge, where SNPs of low quality are discarded.
        This plot shows the number of SNPs that fell in to the different MultiVCFAnalyzer categories:

        * _SNP Calls (all):_ Total number of non-reference homzygous and heterozygous calls made
        * _Filtered SNP call:_ Number of non-reference positions excluded by user-supplied list.
        * _Number of Reference Calls:_ Number of reference calls made
        * _Discarded Reference Calls:_ Number of reference positions not reaching genotyping or coverage thresholds
        * _Discarded SNP Call:_ Number of non-reference positions not reaching enough coverage.
        * _No Call:_ Number of positions with no call made as reported by GATK
        * _Unhandled Genotypes:_ Number of positions where more than two possible alleles occured and were discarded
        """,
        plot = self.addBarplot()
        )


    def parse_data(self, f):
        try:
            data = json.load(f['f'])
        except Exception as e:
            log.debug(e)
            log.warning("Could not parse MultiVCFAnalyzer JSON: '{}'".format(f['fn']))
            return

        # Parse JSON data to a dict
        for s_name, metrics in data.get('metrics', {}).items():
            s_clean = self.clean_s_name(s_name, f['root'])
            if s_clean in self.mvcf_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(s_clean))

            self.add_data_source(f, s_clean)
            self.mvcf_data[s_clean] = dict()
            for snp_prop, value in metrics.items():
                self.mvcf_data[s_clean][snp_prop] = value

    # Compute % heterozygous snp alleles and add to data
    def compute_perc_hets(self):
        """Take the parsed stats from MultiVCFAnalyzer and add one column per sample """
        for sample in self.mvcf_data:
            try:
                self.mvcf_data[sample]['Heterozygous SNP alleles (percent)'] = (self.mvcf_data[sample]['SNP Calls (het)'] / self.mvcf_data[sample]['SNP Calls (all)'])*100
            except ZeroDivisionError:
                self.mvcf_data[sample]['Heterozygous SNP alleles (percent)'] = 'NA'

    def computeSnpHom(self):
        """ Computes snp(hom) for data present by MultiVCFAnalyzer"""
        for sample in self.mvcf_data:
            self.mvcf_data[sample]['SNP Calls (hom)'] = (self.mvcf_data[sample]['SNP Calls (all)'])- self.mvcf_data[sample]['SNP Calls (het)']

    def addSummaryMetrics(self):
        """ Take the parsed stats from MultiVCFAnalyzer and add it to the main plot """

        headers = OrderedDict()

        headers['SNP Calls (all)'] = {
            'title': 'SNPs',
            'description': 'Total number of non-reference calls',
            'scale': 'RdGn',
            'shared_key': 'snp_call',
            'format': '{:,.0f}'
        }
        headers['SNP Calls (hom)'] = {
            'title': 'Hom SNPs',
            'description': 'Total number of non-reference calls passing homozygosity thresholds',
            'scale': 'RdYlGn',
            'hidden': True,
            'shared_key': 'snp_call',
            'format': '{:,.0f}'
        }
        headers['SNP Calls (het)'] = {
            'title': 'Het SNPs',
            'description': 'Total number of non-reference calls not passing homozygosity thresholds',
            'scale': 'YlGn',
            'hidden': True,
            'shared_key': 'snp_call',
            'format': '{:,.0f}'
        }
        headers['Heterozygous SNP alleles (percent)'] = {
            'title': '% Hets',
            'description': 'Percentage of heterozygous SNP alleles',
            'scale': 'OrRd',
            'max': 100,
            'min': 0
        }
        self.general_stats_addcols(self.mvcf_data, headers)

    def addTable(self):
        """ Take the parsed stats from MultiVCFAnalyzer and add it to the MVCF Table"""
        headers = OrderedDict()

        headers['allPos'] = {
            'title': 'Bases in Final Alignment',
            'description': 'Length of FASTA file in base pairs (bp)',
            'scale': 'BrBG',
            'shared_key': 'calls',
            'format': '{:,.0f}'
        }
        headers['SNP Calls (all)'] = {
            'title': 'SNPs',
            'description': 'Total number of non-reference calls made',
            'scale': 'OrRd',
            'shared_key': 'snp_call',
            'format': '{:,.0f}'
        }
        headers['Heterozygous SNP alleles (percent)'] = {
            'title': '% Hets',
            'description': 'Percentage of heterozygous SNP alleles',
            'scale': 'PuBu',
            'max': 100,
            'min': 0
        }
        headers['SNP Calls (hom)'] = {
            'title': 'Hom SNPs',
            'description': 'Total number of non-reference calls passing homozygosity thresholds',
            'scale': 'RdYlGn',
            'shared_key': 'snp_call',
            'format': '{:,.0f}'
        }
        headers['SNP Calls (het)'] = {
            'title': 'Het SNPs',
            'description': 'Total number of non-reference calls not passing homozygosity thresholds',
            'scale': 'RdYlGn',
            'shared_key': 'snp_call',
            'format': '{:,.0f}'
        }
        headers['discardedVarCall'] = {
            'title': 'Discarded SNP Call',
            'description': 'Number of non-reference positions not reaching genotyping or coverage thresholds',
            'scale': 'PuCr',
            'shared_key': 'calls',
            'format': '{:,.0f}'
        }
        headers['filteredVarCall'] = {
            'title': 'Filtered SNP Call',
            'description': 'Number of positions ignored defined in user-supplied filter list',
            'scale': 'RdGy',
            'shared_key': 'calls',
            'format': '{:,.0f}'
        }
        headers['refCall'] = {
            'title': 'Reference Calls',
            'description': 'Number of reference calls made',
            'scale': 'Spectral',
            'shared_key': 'calls',
            'format': '{:,.0f}'
        }
        headers['discardedRefCall'] = {
            'title': 'Discarded Reference Call',
            'description': 'Number of reference positions not reaching genotyping or coverage thresholds',
            'scale': 'YlGnBu',
            'shared_key': 'calls',
            'format': '{:,.0f}'
        }
        headers['coverage (fold)'] = {
            'title': 'Average Call Coverage',
            'description': 'Average number of reads covering final calls',
            'scale': 'OrRd',
            'shared_key': 'coverage',
            'suffix': 'X'
        }
        headers['coverage (percent)'] = {
            'title': '% Reference with Calls',
            'description': 'Percent coverage of all positions with final calls',
            'scale': 'PuBuGn',
            'shared_key': 'coverage',
            'suffix': '%',
            'max': 100,
            'min': 0
        }
        headers['unhandledGenotype'] = {
            'title': 'Unhandled Genotypes',
            'description': 'Number of positions discarded due to presence of more than one alternate allele',
            'scale': 'BuPu',
            'shared_key': 'snp_count',
            'format': '{:,.0f}'
        }
        headers['noCall'] = {
            'title': 'Positions with No Call',
            'description': 'Number of positions with no call made as reported by GATK',
            'scale': 'GnBu',
            'shared_key': 'calls',
            'format': '{:,.0f}'
        }

        # Separate table config
        table_config = {
        'namespace': 'MultiVCFAnalyzer',                         # Name for grouping. Prepends desc and is in Config Columns modal
        'id': 'mvcf-table',                 # ID used for the table
        'table_title': 'MultiVCFAnalyzer Results',             # Title of the table. Used in the column config modal
        }
        tab = table.plot(self.mvcf_data, headers, table_config)
        return tab



    def addBarplot(self):
        """ Take the parsed stats from MultiVCFAnalyzer and add it to the MVCF Table"""
        cats = OrderedDict()


    # SNP Calls (all): Green CHECK
    # Number of Reference Calls: Blue CHECK
    # Discarded SNP Call: Orange CHECK
    # Discarded Reference Call: Red [if avaliable] CHECK
    # Positions with No Call: Black CHECK
        cats['SNP Calls (hom)'] = {
            'name': 'SNP Calls (hom)',
            'color': '#01665e'
        }
        cats['SNP Calls (het)'] = {
            'name': 'SNP Calls (het)',
            'color': '#5ab4ac'
        }
        cats['refCall'] = {
            'name': 'Reference Calls',
            'color': '#9ebcda'
        }
        cats['discardedVarCall'] = {
            'name': 'Discarded SNP Call',
            'color': '#f7a35c'
        }
        cats['filteredVarCall'] = {
            'name': 'Filtered SNP Call',
        }
        cats['discardedRefCall'] = {
            'name': 'Discarded Reference Call',
            'color': '#e34a33'
        }
        cats['unhandledGenotype'] = {
            'name': 'Positions with an unknown Genotype Call',
            'color': '#252525'
        }
        cats['noCall'] = {
            'name': 'Positions with No Call',
            'color': '#252525'
        }

        config = {
        # Building the plot
        'id': 'mvcf_barplot',                # HTML ID used for plot
        'hide_zero_cats': True,                 # Hide categories where data for all samples is 0
        # Customising the plot
        'title': 'MultiVCFAnalyzer: Call Categories',                          # Plot title - should be in format "Module Name: Plot Title"
        'ylab': 'Total # Positions',                           # X axis label
        'xlab': None,
        'stacking': 'normal',                   # Set to None to have category bars side by side
        'use_legend': True,                     # Show / hide the legend
        }
        return bargraph.plot(self.mvcf_data, cats, config)



#!/usr/bin/env python

""" MultiQC module to parse output from Flexbar """

from __future__ import print_function
from collections import OrderedDict
import logging
import re

from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Flexbar', anchor='flexbar',
        href='https://github.com/seqan/flexbar',
        info="is a barcode and adapter removal tool.")

        # Parse logs
        self.flexbar_data = dict()
        for f in self.find_log_files('flexbar', filehandles=True):
            self.parse_flexbar(f)

        # Filter to strip out ignored sample names
        self.flexbar_data = self.ignore_samples(self.flexbar_data)

        if len(self.flexbar_data) == 0:
            raise UserWarning

        log.info("Found {} logs".format(len(self.flexbar_data)))
        self.write_data_file(self.flexbar_data, 'multiqc_flexbar')

        # Add drop rate to the general stats table
        headers = {}
        headers['removed_bases_pct'] = {
            'title': '% bp Trimmed',
            'description': '% Total Base Pairs removed',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'YlOrRd'
        }
        self.general_stats_addcols(self.flexbar_data, headers)

        # Make barplot
        self.flexbar_barplot()

    def parse_flexbar(self, f):

        def _save_data(parsed_data):
            if len(parsed_data) > 0:
                # Calculate removed_bases
                if 'processed_bases' in parsed_data and 'remaining_bases' in parsed_data:
                    parsed_data['removed_bases'] = parsed_data['processed_bases'] - parsed_data['remaining_bases']
                    parsed_data['removed_bases_pct'] = (float(parsed_data['removed_bases']) / float(parsed_data['processed_bases']) ) * 100.0
                if s_name in self.flexbar_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                self.flexbar_data[s_name] = parsed_data

        regexes = {
            'output_filename': r"Read file:\s+(.+)$",
            'processed_reads': r"Processed reads\s+(\d+)",
            'skipped_due_to_uncalled_bases': r"skipped due to uncalled bases\s+(\d+)",
            'short_prior_to_adapter_removal': r"short prior to adapter removal\s+(\d+)",
            'finally_skipped_short_reads': r"finally skipped short reads\s+(\d+)",
            'discarded_reads_overall': r"Discarded reads overall\s+(\d+)",
            'remaining_reads': r"Remaining reads\s+(\d+)",
            'processed_bases': r"Processed bases:?\s+(\d+)",
            'remaining_bases': r"Remaining bases:?\s+(\d+)",
        }
        s_name = f['s_name']
        parsed_data = dict()
        for l in f['f']:
            for k, r in regexes.items():
                match = re.search(r, l)
                if match:
                    if k == 'output_filename':
                        s_name = self.clean_s_name(match.group(1), f['root'])
                    else:
                        parsed_data[k] = int(match.group(1))

            # End of log output. Save and reset in case of more logs.
            if 'Flexbar completed' in l:
                _save_data(parsed_data)
                s_name = f['s_name']
                parsed_data = dict()

        # Pick up any partial logs
        _save_data(parsed_data)

    def flexbar_barplot (self):
        """ Make the HighCharts HTML to plot the flexbar rates """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['remaining_reads'] =                { 'color': '#437bb1', 'name': 'Remaining reads' }
        keys['skipped_due_to_uncalled_bases'] =  { 'color': '#e63491', 'name': 'Skipped due to uncalled bases' }
        keys['short_prior_to_adapter_removal'] = { 'color': '#b1084c', 'name': 'Short prior to adapter removal' }
        keys['finally_skipped_short_reads'] =    { 'color': '#7f0000', 'name': 'Finally skipped short reads' }

        # Config for the plot
        pconfig = {
            'id': 'flexbar_plot',
            'title': 'Flexbar: Processed Reads',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads',
            'hide_zero_cats': False
        }

        self.add_section( plot = bargraph.plot(self.flexbar_data, keys, pconfig) )



#!/usr/bin/env python

""" MultiQC module to parse TsTv by summary output from vcftools TsTv-summary """

import logging
from collections import OrderedDict
from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)

class TsTvSummaryMixin():

    def parse_tstv_summary(self):
        """ Create the HTML for the TsTv summary plot. """

        self.vcftools_tstv_summary = dict()
        for f in self.find_log_files('vcftools/tstv_summary', filehandles=True):
            d = {}
            for line in f['f'].readlines()[1:]: # don't add the header line (first row)
                key = line.split()[0] # taking the first column (MODEL) as key
                val = int(line.split()[1]) # taking the second column (COUNT) as value
                d[key] = val
            self.vcftools_tstv_summary[f['s_name']] = d

        # Filter out ignored sample names
        self.vcftools_tstv_summary = self.ignore_samples(self.vcftools_tstv_summary)

        if len(self.vcftools_tstv_summary) == 0:
            return 0

        # Specifying the categories of the bargraph
        keys = OrderedDict()
        keys = ['AC', 'AG', 'AT', 'CG', 'CT', 'GT', 'Ts', 'Tv']

        pconfig = {
            'id': 'vcftools_tstv_summary',
            'title': 'VCFTools: TsTv Summary',
            'ylab': 'Counts',
        }

        self.add_section(
            name = 'TsTv Summary',
            anchor = 'vcftools-tstv-summary',
            description = "Plot of `TSTV-SUMMARY` - count of different types of transition and transversion SNPs.",
            plot = bargraph.plot(self.vcftools_tstv_summary,keys,pconfig)
        )

        return len(self.vcftools_tstv_summary)




#!/usr/bin/env python
from __future__ import print_function
import logging
from collections import OrderedDict

from multiqc.plots import table
from multiqc.modules.base_module import BaseMultiqcModule

from .bbmap_filetypes import file_types, section_order

""" MultiQC module to parse output from BBMap """

# Initialize the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ BBMap module, tries to identify and parse tons of output files
    generated by BBMap.
    """

    def __init__(self):
        super(MultiqcModule, self).__init__(
            name="BBTools",
            anchor="bbmap",
            href="http://jgi.doe.gov/data-and-tools/bbtools/",
            info="""is a suite of fast multithreaded bioinformatics tools
            designed for the analysis of DNA and RNA sequence data."""
        )

        # Init data dict
        self.mod_data = { key:{} for key in file_types }

        # Find output files
        module_filetypes = [('bbmap/'+ft, ft) for ft in file_types]
        data_found = False
        for module_filetype, file_type in module_filetypes:
            for f in self.find_log_files(module_filetype, filehandles=True):
                if self.parse_logs(file_type, **f):
                    self.add_data_source(f)
                    data_found = True

        if not data_found:
            raise UserWarning
        else:
            num_samples = max([len(self.mod_data[ft].keys()) for ft in self.mod_data])
            log.info("Found {} reports".format(num_samples))

        for file_type in section_order:
            if len(self.mod_data[file_type]) > 0:
                log.debug("section %s has %d entries", file_type,
                          len(self.mod_data[file_type]))

                if file_types[file_type]['plot_func']:
                    self.add_section(
                        name = file_types[file_type]['title'],
                        anchor =  'bbmap-' + file_type,
                        description = file_types[file_type]['descr'],
                        helptext = file_types[file_type]['help_text'],
                        plot = self.plot(file_type)
                    )

            if any(self.mod_data[file_type][sample]['kv']
                   for sample in self.mod_data[file_type]):
                self.add_section(
                    name = file_types[file_type]['title']+' summary table',
                    anchor =  'bbmap-' + file_type + '-table',
                    description = file_types[file_type]['descr'],
                    helptext = file_types[file_type]['help_text'],
                    plot = self.make_basic_table(file_type)
                )


    def parse_logs(self, file_type, root, s_name, fn, f, **kw):

        if self.is_ignore_sample(s_name):
            return False

        log.debug("Parsing %s/%s", root, fn)
        if not file_type in file_types:
            log.error("Unknown output type '%s'. Error in config?", file_type)
            return False
        log_descr = file_types[file_type]
        if 'not_implemented' in log_descr:
            log.debug("Can't parse '%s' -- implementation missing", file_type)
            return False

        cols = log_descr['cols']
        if isinstance(cols, OrderedDict):
            cols = list(cols.keys())

        kv = {}
        data = {}
        for line_number, line in enumerate(f, start=1):
            line = line.strip().split('\t')
            try:
                header_row = line[0][0] == '#'
            except IndexError:
                continue  # The table is probably empty
            if header_row:
                line[0] = line[0][1:] # remove leading '#'

                if line[0] != cols[0]:
                    # It's not the table header, it must be a key-value row
                    if len(line) == 3 and file_type == "stats":
                        # This is a special case for the 'stats' file type:
                        # The first line _might_ have three columns if processing paired-end reads,
                        # but we don't care about the first line.
                        # The third line is always three columns, which is what we really want.
                        if line[0] == "File":
                            continue
                        kv["Percent filtered"] = float(line[2].strip("%"))
                        kv[line[0]] = line[1]
                    elif len(line) != 2:
                        # Not two items? Wrong!
                        log.error("Expected key value pair in %s/%s:%d but found '%s'",
                                  root, s_name, line_number, repr(line))
                        log.error("Table header should begin with '%s'",
                                  cols[0])
                    else:
                        # save key value pair
                        kv[line[0]] = line[1]
                else:
                    # It should be the table header. Verify:
                    if line != cols:
                        if line != cols + list(log_descr['extracols'].keys()):
                            log.error("Table headers do not match those 'on file'. %s != %s",
                                      repr(line), repr(cols))
                        return False
            else:
                if isinstance(log_descr['cols'], OrderedDict):
                    line = [
                        value_type(value)
                        for value_type, value in zip(log_descr['cols'].values(), line)
                    ]
                else:
                    line = list(map(int, line))
                data[line[0]] = line[1:]

        if not data:
            log.warning("File %s appears to contain no data for plotting, ignoring...", fn)
            return False

        if s_name in self.mod_data[file_type]:
            log.debug("Duplicate sample name found! Overwriting: %s", s_name)

        self.mod_data[file_type][s_name] = {'data':data, 'kv': kv}
        log.debug("Found %s output for sample %s with %d rows",
                  file_type, s_name, len(data))

        return True

    def plot(self, file_type):
        """ Call file_type plotting function.
        """

        samples = self.mod_data[file_type]
        plot_title = file_types[file_type]['title']
        plot_func = file_types[file_type]['plot_func']
        plot_params = file_types[file_type]['plot_params']
        return plot_func(samples,
                    file_type,
                    plot_title=plot_title,
                    plot_params=plot_params)


    def make_basic_table(self, file_type):
        """  Create table of key-value items in 'file_type'.
        """

        table_data = {sample: items['kv']
                for sample, items
                in self.mod_data[file_type].items()
        }
        table_headers = {}
        for column_header, (description, header_options) in file_types[file_type]['kv_descriptions'].items():
            table_headers[column_header] = {
                    'rid': '{}_{}_bbmstheader'.format(file_type, column_header),
                    'title': column_header,
                    'description': description,
            }
            table_headers[column_header].update(header_options)

        tconfig = {
            'id': file_type + '_bbm_table',
            'namespace': 'BBTools'
        }
        for sample in table_data:
            for key, value in table_data[sample].items():
                try:
                    table_data[sample][key] = float(value)
                except ValueError:
                    pass
        return table.plot(table_data, table_headers, tconfig)



#!/usr/bin/env python

""" MultiQC module to parse output from FastQ Screen """

from __future__ import print_function
from collections import OrderedDict
import json
import logging
import re

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.utils import report

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='FastQ Screen', anchor='fastq_screen',
        href="http://www.bioinformatics.babraham.ac.uk/projects/fastq_screen/",
        info="allows you to screen a library of sequences in FastQ format against"\
        " a set of sequence databases so you can see if the composition of the"\
        " library matches with what you expect.")

        # Find and load any FastQ Screen reports
        self.fq_screen_data = dict()
        self.num_orgs = 0
        for f in self.find_log_files('fastq_screen', filehandles=True):
            parsed_data = self.parse_fqscreen(f)
            if parsed_data is not None:
                if f['s_name'] in self.fq_screen_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
                self.add_data_source(f)
                self.fq_screen_data[f['s_name']] = parsed_data

        # Filter to strip out ignored sample names
        self.fq_screen_data = self.ignore_samples(self.fq_screen_data)

        if len(self.fq_screen_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.fq_screen_data)))

        # Check whether we have a consistent number of organisms across all samples
        num_orgs = set([len(orgs) for orgs in self.fq_screen_data.values()])

        # Section 1 - Alignment Profiles
        # Posh plot only works for around 20 samples, 8 organisms. If all samples have the same number of organisms.
        if len(num_orgs) == 1 and len(self.fq_screen_data) * self.num_orgs <= 160 and not config.plots_force_flat and not getattr(config, 'fastqscreen_simpleplot', False):
            self.add_section(
                name = 'Mapped Reads',
                anchor = 'fastq_screen_mapped_reads',
                content = self.fqscreen_plot()
            )
        # Use simpler plot that works with many samples
        else:
            self.add_section(
                name = 'Mapped Reads',
                anchor = 'fastq_screen_mapped_reads',
                plot = self.fqscreen_simple_plot()
            )

        # Section 2 - Optional bisfulfite plot
        self.fqscreen_bisulfite_plot()

        # Write the total counts and percentages to files
        self.write_data_file(self.parse_csv(), 'multiqc_fastq_screen')


    def parse_fqscreen(self, f):
        """ Parse the FastQ Screen output into a 3D dict """
        parsed_data = OrderedDict()
        nohits_pct = None
        headers = None
        bs_headers = None
        for l in f['f']:
            # Skip comment lines
            if l.startswith('#'):
                continue
            if l.startswith('%Hit_no_genomes:') or l.startswith('%Hit_no_libraries:'):
                nohits_pct = float(l.split(':', 1)[1])
                parsed_data['No hits'] = {'percentages': {'one_hit_one_library': nohits_pct }}
            else:
                s = l.strip().split("\t")

                # Regular FastQ Screen table section
                if len(s) == 12:
                    if headers is None:
                        headers = s
                    else:
                        # Can't use #Reads in subset as varies. #Reads_processed should be same for all orgs in a sample
                        parsed_data['total_reads'] = int(s[1])
                        # Loop through all columns
                        parsed_data[s[0]] = {'percentages':{}, 'counts':{}}
                        for idx, h in enumerate(headers):
                            if idx == 0:
                                continue
                            dtype = 'percentages' if h.startswith('%') else 'counts'
                            field = h.replace('%', '').replace('#', '').replace('genomes', 'libraries').replace('genome', 'library').lower()
                            parsed_data[s[0]][dtype][field] = float(s[idx])

                # Optional Bisulfite table section
                elif len(s) == 9:
                    if bs_headers is None:
                        bs_headers = s
                    else:
                        # Loop through all columns
                        parsed_data[s[0]]['bisulfite_percentages'] = {}
                        parsed_data[s[0]]['bisulfite_counts'] = {}
                        for idx, h in enumerate(bs_headers):
                            if idx == 0:
                                continue
                            dtype = 'bisulfite_percentages' if h.startswith('%') else 'bisulfite_counts'
                            field = h.replace('%', '').replace('#', '').lower()
                            parsed_data[s[0]][dtype][field] = float(s[idx])

        if len(parsed_data) == 0:
            return None

        # Calculate no hits counts
        if parsed_data.get('total_reads') is not None and nohits_pct is not None:
            parsed_data['No hits']['counts'] = {'one_hit_one_library': int((nohits_pct/100.0) * float(parsed_data['total_reads'])) }
        else:
            log.warning("Couldn't find number of reads with no hits for '{}'".format(f['s_name']))

        self.num_orgs = max(len(parsed_data), self.num_orgs)
        return parsed_data

    def parse_csv(self):
        totals = OrderedDict()
        for s in sorted(self.fq_screen_data.keys()):
            totals[s] = OrderedDict()
            for org in self.fq_screen_data[s]:
                if org == 'total_reads':
                    totals[s]['total_reads'] = self.fq_screen_data[s][org]
                    continue
                try:
                    k = "{} counts".format(org)
                    totals[s][k] = self.fq_screen_data[s][org]['counts']['one_hit_one_library']
                    totals[s][k] += self.fq_screen_data[s][org]['counts'].get('multiple_hits_one_library', 0)
                    totals[s][k] += self.fq_screen_data[s][org]['counts'].get('one_hit_multiple_libraries', 0)
                    totals[s][k] += self.fq_screen_data[s][org]['counts'].get('multiple_hits_multiple_libraries', 0)
                except KeyError: pass
                try:
                    k = "{} percentage".format(org)
                    totals[s][k] = self.fq_screen_data[s][org]['percentages']['one_hit_one_library']
                    totals[s][k] += self.fq_screen_data[s][org]['percentages'].get('multiple_hits_one_library', 0)
                    totals[s][k] += self.fq_screen_data[s][org]['percentages'].get('one_hit_multiple_libraries', 0)
                    totals[s][k] += self.fq_screen_data[s][org]['percentages'].get('multiple_hits_multiple_libraries', 0)
                except KeyError: pass
        return totals

    def fqscreen_plot (self):
        """ Makes a fancy custom plot which replicates the plot seen in the main
        FastQ Screen program. Not useful if lots of samples as gets too wide. """

        categories = list()
        getCats = True
        data = list()
        p_types = OrderedDict()
        p_types['multiple_hits_multiple_libraries'] = {'col': '#7f0000', 'name': 'Multiple Hits, Multiple Genomes' }
        p_types['one_hit_multiple_libraries'] = {'col': '#ff0000', 'name': 'One Hit, Multiple Genomes' }
        p_types['multiple_hits_one_library'] = {'col': '#00007f', 'name': 'Multiple Hits, One Genome' }
        p_types['one_hit_one_library'] = {'col': '#0000ff', 'name': 'One Hit, One Genome' }
        for k, t in p_types.items():
            first = True
            for s in sorted(self.fq_screen_data.keys()):
                thisdata = list()
                if len(categories) > 0:
                    getCats = False
                for org in sorted(self.fq_screen_data[s]):
                    if org == 'total_reads':
                        continue
                    try:
                        thisdata.append(self.fq_screen_data[s][org]['percentages'][k])
                    except KeyError:
                        thisdata.append(None)
                    if getCats:
                        categories.append(org)
                td = {
                    'name': t['name'],
                    'stack': s,
                    'data': thisdata,
                    'color': t['col']
                }
                if first:
                    first = False
                else:
                    td['linkedTo'] = ':previous'
                data.append(td)

        plot_id = report.save_htmlid('fq_screen_plot')
        html = '''<div id={plot_id} class="fq_screen_plot hc-plot"></div>
        <script type="application/json" class="fq_screen_dict">{dict}</script>
        '''.format(
            plot_id=json.dumps(plot_id),
            dict=json.dumps({ 'plot_id': plot_id, 'data': data, 'categories': categories }),
        )

        html += '''<script type="text/javascript">
            fq_screen_dict = { }; // { <plot_id>: data, categories }
            $('.fq_screen_dict').each(function (i, elem) {
                var dict = JSON.parse(elem.innerHTML);
                fq_screen_dict[dict.plot_id] = dict;
            });

            $(function () {
                // In case of repeated modules: #fq_screen_plot, #fq_screen_plot-1, ..
                $(".fq_screen_plot").each(function () {
                    var plot_id = $(this).attr('id');

                    $(this).highcharts({
                        chart: { type: "column", backgroundColor: null },
                        title: { text: "FastQ Screen Results" },
                        xAxis: { categories: fq_screen_dict[plot_id].categories },
                        yAxis: {
                            max: 100,
                            min: 0,
                            title: { text: "Percentage Aligned" }
                        },
                        tooltip: {
                            formatter: function () {
                                return "<b>" + this.series.stackKey.replace("column","") + " - " + this.x + "</b><br/>" +
                                    this.series.name + ": " + this.y + "%<br/>" +
                                    "Total Alignment: " + this.point.stackTotal + "%";
                            },
                        },
                        plotOptions: {
                            column: {
                                pointPadding: 0,
                                groupPadding: 0.02,
                                stacking: "normal"
                            }
                        },
                        series: fq_screen_dict[plot_id].data
                    });
                });
            });
        </script>'''

        return html


    def fqscreen_simple_plot(self):
        """ Makes a simple bar plot with summed alignment counts for
        each species, stacked. """

        # First, sum the different types of alignment counts
        data = {}
        org_counts = {}
        for s_name in sorted(self.fq_screen_data):
            data[s_name] = OrderedDict()
            sum_alignments = 0
            for org in self.fq_screen_data[s_name]:
                if org == 'total_reads':
                    continue
                try:
                    data[s_name][org] = self.fq_screen_data[s_name][org]['counts']['one_hit_one_library']
                except KeyError:
                    log.error("No counts found for '{}' ('{}'). Could be malformed or very old FastQ Screen results.".format(org, s_name))
                    continue
                try:
                    data[s_name][org] += self.fq_screen_data[s_name][org]['counts']['multiple_hits_one_library']
                except KeyError:
                    pass
                sum_alignments += data[s_name][org]
                org_counts[org] = org_counts.get(org, 0) + data[s_name][org]

            # Calculate hits in multiple genomes
            if 'total_reads' in self.fq_screen_data[s_name]:
                data[s_name]['Multiple Genomes'] = self.fq_screen_data[s_name]['total_reads'] - sum_alignments

        # Sort the categories by the total read counts
        cats = OrderedDict()
        for org in sorted(org_counts, key=org_counts.get, reverse=True):
            if org not in cats and org != 'No hits':
                cats[org] = { 'name': org }


        # Strip empty dicts
        for s_name in list(data.keys()):
            if len(data[s_name]) == 0:
                del data[s_name]

        pconfig = {
            'id': 'fastq_screen_plot',
            'title': 'FastQ Screen: Mapped Reads',
            'cpswitch_c_active': False,
            'ylab': 'Percentages'
        }
        cats['Multiple Genomes'] = { 'name': 'Multiple Genomes', 'color': '#820000' }
        cats['No hits'] = { 'name': 'No hits', 'color': '#cccccc' }

        return bargraph.plot(data, cats, pconfig)

    def fqscreen_bisulfite_plot(self):
        """ Make a stacked barplot for the bisulfite data, if we have any """

        pconfig = {
            'id': 'fastq_screen_bisulfite_plot',
            'title': 'FastQ Screen: Bisulfite Mapping Strand Orientation',
            'hide_zero_cats': False,
            'ylab': 'Percentages',
            'data_labels': []
        }

        cats = OrderedDict()
        cats['original_top_strand'] = { 'name': 'Original top strand', 'color': '#80cdc1' }
        cats['complementary_to_original_top_strand'] = { 'name': 'Complementary to original top strand', 'color': '#018571' }
        cats['complementary_to_original_bottom_strand'] = { 'name': 'Complementary to original bottom strand', 'color': '#a6611a' }
        cats['original_bottom_strand'] = { 'name': 'Original bottom strand', 'color': '#dfc27d' }

        # Pull out the data that we want
        pdata_unsorted = {}
        org_counts = {}
        max_count = 0
        for s_name, d in self.fq_screen_data.items():
            for org, dd in d.items():
                if org not in pdata_unsorted:
                    pdata_unsorted[org] = {}
                    org_counts[org] = 0
                try:
                    pdata_unsorted[org][s_name] = dd['bisulfite_counts']
                    total_counts = sum([c for c in dd['bisulfite_counts'].values()])
                    org_counts[org] += total_counts
                    max_count = max(max_count, total_counts)
                except (TypeError, KeyError):
                    pass

        # Consistent y-max across all genomes
        pconfig['ymax'] = max_count

        # Show tabbed bar plots, sorted by total read count
        pdata = []
        pcats = []
        for org in sorted(org_counts, key=org_counts.get, reverse=True):
            if org_counts[org] > 0:
                pconfig['data_labels'].append(org)
                pdata.append(pdata_unsorted[org])
                pcats.append(cats)

        if len(pdata) == 0:
            return None



        self.add_section(
            name = 'Bisulfite Reads',
            anchor = 'fastq_screen_bisulfite',
            plot = bargraph.plot(pdata, pcats, pconfig)
        )



#!/usr/bin/env python
""" MultiQC module to parse output from MALT """
from __future__ import print_function
from collections import OrderedDict
from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

import logging
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    """Malt Module"""

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name='MALT',
            anchor='malt',
            href='https://uni-tuebingen.de/fakultaeten/mathematisch-naturwissenschaftliche-fakultaet/fachbereiche/informatik/lehrstuehle/algorithms-in-bioinformatics/software/malt/',
            info='performs alignment of metagenomic reads against a database of reference sequences (such as NR, GenBank or Silva) and produces a MEGAN RMA file as output.'
        )

        # Find and load Malt reports
        self.malt_data = dict()

        for f in self.find_log_files('malt', filehandles=True):
            self.parse_logs(f)

        self.malt_data = self.ignore_samples(self.malt_data)

        if len(self.malt_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.malt_data)))

        self.malt_general_stats()
        self.mappability_barplot()
        self.taxonomic_assignation_barplot()

    def parse_logs(self, f):
        """Parses a Malt log file"""
        reading = False
        keys = [
            "Total reads",
            "Assig. Taxonomy",
            "Num. of queries",
            "Aligned queries",
            "Num. alignments",
        ]
        for line in f['f']:
            line = line.rstrip()
            if line.startswith("+++++ Aligning file:") and reading == False:
                reading = True
                s_name = line.split()[-1]
                s_name = self.clean_s_name(s_name, f['root'])
                if s_name in self.malt_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                self.add_data_source(f, s_name=s_name)
                self.malt_data[s_name] = {}
            elif reading:
                for k in keys:
                    if line.startswith(k):
                        self.malt_data[s_name][k] = int(line.split()[-1].replace(",", ""))
                        if k == 'Num. alignments':
                            try:
                                self.malt_data[s_name]['Non mapped'] = self.malt_data[s_name]["Num. of queries"] - self.malt_data[s_name]['Total reads']
                                self.malt_data[s_name]['No Assig. Taxonomy'] = self.malt_data[s_name]["Total reads"] - self.malt_data[s_name]['Assig. Taxonomy']
                                self.malt_data[s_name]['Mappability'] = (float(self.malt_data[s_name]['Total reads']) / float(self.malt_data[s_name]["Num. of queries"]))*100.0
                                self.malt_data[s_name]['Taxonomic assignment success'] = (float(self.malt_data[s_name]['Assig. Taxonomy']) / float(self.malt_data[s_name]['Total reads']))*100.0
                            except KeyError:
                                pass
                            reading = False

    def mappability_barplot(self):
        """Mappability barplot

        Mappability = (Total reads / Num. of queries) * 100
        """
        cats = OrderedDict()
        cats['Total reads'] = {'name': 'Mapped reads'}
        cats['Non mapped'] = {'name': 'Non Mapped reads'}
        config = {
            'id': 'malt-mappability-plot',
            'title': 'MALT: Metagenomic Mappability',
            'ylab': 'Samples',
        }
        self.add_section(
            name='Metagenomic Mappability',
            anchor='malt-mappability',
            description='Number of mapped reads.',
            plot=bargraph.plot(self.malt_data, cats, config)
        )

    def taxonomic_assignation_barplot(self):
        """Taxonomic assignment barplot

        Taxonomic assignment success = (Assig. Taxonomy / Total reads) * 100
        """
        cats = ['Assig. Taxonomy', "No Assig. Taxonomy"]
        config = {
            'id': 'malt-taxonomic-success-plot',
            'title': 'MALT: Taxonomic assignment success',
            'ylab': 'Samples',
        }
        self.add_section(
            name='Taxonomic assignment success',
            anchor='malt-taxonomic-success',
            description='Shows the number of mapped reads assigned to a taxonomic node.',
            plot=bargraph.plot(self.malt_data, cats, config)
        )

    def malt_general_stats(self):
        """MALT General Statistics table"""
        headers = OrderedDict()
        headers['Taxonomic assignment success'] = {
            'title': "% Tax assigned",
            'description': 'Percentage of mapped reads assigned to a taxonomic node',
            'suffix': '%',
            'max': 100,
            'scale': 'RdYlGn'
        }
        headers['Assig. Taxonomy'] = {
            'title': '{} Tax assigned'.format(config.read_count_prefix),
            'description': 'Number of reads assigned to a Taxonomic node ({})'.format(config.read_count_desc),
            'scale': 'Greens',
            'shared_key': 'read_count',
            'modify': lambda x: x * config.read_count_multiplier,
            'hidden': True
        }
        headers['Mappability'] = {
            'title': "% Metagenomic Mapped",
            'description': 'Percentage of mapped reads',
            'suffix': '%',
            'max': 100,
            'scale': 'RdYlGn'
        }
        headers['Total reads'] = {
            'title': '{} Mapped'.format(config.read_count_prefix),
            'description': 'Number of mapped reads ({})'.format(config.read_count_desc),
            'scale': 'PuBu',
            'shared_key': 'read_count',
            'modify': lambda x: x * config.read_count_multiplier,
        }
        headers['Num. of queries'] = {
            'title': '{} Reads'.format(config.read_count_prefix),
            'description': 'Number of reads in sample ({})'.format(config.read_count_desc),
            'scale': 'Purples',
            'shared_key': 'read_count',
            'modify': lambda x: x * config.read_count_multiplier,
        }
        self.general_stats_addcols(self.malt_data, headers)



#!/usr/bin/env python

""" MultiQC submodule to parse output from MinIONQC summary stats """

from collections import OrderedDict
import copy
import yaml
import os
import re
import logging

from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.plots import linegraph, table

# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name='MinIONQC',
            anchor='minionqc',
            href="https://github.com/roblanf/minion_qc",
            info=" is a QC tool for Oxford Nanopore sequencing data"
        )

        # Find and load any minionqc reports
        self.minionqc_raw_data = dict() # main dataset in original YAML format
        self.minionqc_data = dict()     # main dataset. Stats from all reads
        self.qfilt_data = dict()        # Stats from quality filtered reads
        self.q_threshold_list = set()        # quality thresholds
        for f in self.find_log_files('minionqc', filehandles=True):

            # get sample name
            s_name = self.clean_s_name(os.path.basename(f['root']),os.path.dirname(f['root']))

            # parses minionqc summary data
            parsed_dict = self.parse_minionqc_report(s_name, f['f'])

            if parsed_dict is not None:
                if s_name in self.minionqc_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))

                # adds files used in MultiQC report
                self.add_data_source(f, s_name)


        # Filter to strip out ignored sample names
        self.minionqc_data = self.ignore_samples(self.minionqc_data)
        if len(self.minionqc_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.minionqc_data)))

        # columns to present in MultiQC summary table
        headers = self.headers_to_use()

        # Just the total number of reads in the main General Statistics table
        self.general_stats_addcols(self.minionqc_data, {'total.reads': headers['total.reads']})

        # writes parsed data into a file (by MultiQC)
        self.write_data_file(self.minionqc_data, 'multiqc_minionqc')

        # tables and plots in order
        self.table_qALL()
        self.table_qfiltered()
        self.plot_readlengths()


    def parse_minionqc_report(self, s_name, f):
        '''
        Parses minionqc's 'summary.yaml' report file for results.
        Uses only the "All reads" stats. Ignores "Q>=x" part.
        '''
        try:
            # Parsing as OrderedDict is slightly messier with YAML
            # http://stackoverflow.com/a/21048064/713980
            def dict_constructor(loader, node):
                return OrderedDict(loader.construct_pairs(node))
            yaml.add_constructor(yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG, dict_constructor)
            summary_dict = yaml.safe_load(f)
        except Exception as e:
            log.error("Error parsing MinIONQC input file: {}".format(f))
            return

        # Do a deep copy as dicts are immutable
        self.minionqc_raw_data[s_name] = copy.deepcopy(summary_dict)

        # get q value threshold used for reads
        q_threshold = None
        for k in summary_dict.keys():
            if k.startswith('Q>='):
                q_threshold = k

        data_dict = {}
        data_dict['all'] = summary_dict['All reads']        # all reads
        data_dict['q_filt'] = summary_dict[q_threshold]     # quality filtered reads

        for q_key in ['all', 'q_filt']:
            for key_1 in ['reads', 'gigabases']:
                for key_2 in data_dict[q_key][key_1]:
                    new_key = '{} {}'.format(key_1, key_2)
                    data_dict[q_key][new_key] = data_dict[q_key][key_1][key_2]
                data_dict[q_key].pop(key_1)    # removes key after flattening

        self.minionqc_data[s_name] = data_dict['all']     # stats for all reads
        self.qfilt_data[s_name] = data_dict['q_filt']     # stats for q-filtered reads
        self.q_threshold_list.add(q_threshold)            # quality threshold used in this file


    def headers_to_use(self):
        '''
        Defines features of columns to be used in multiqc table
        '''
        headers = OrderedDict()

        headers['total.reads'] = {
            'title': 'Total reads',
            'description': 'Total number of reads',
            'format': '{:,.0f}',
            'scale': 'Greys'
        }
        headers['total.gigabases'] = {
            'title': 'Total bases (GB)',
            'description': 'Total bases',
            'format': '{:,.2f}',
            'scale': 'Blues'
        }
        headers['N50.length'] = {
            'title': 'Reads N50',
            'description': 'Minimum read length needed to cover 50% of all reads',
            'format': '{:,.0f}',
            'scale': 'Purples',
        }
        headers['mean.q'] = {
            'title': 'Mean Q score',
            'description': 'Mean quality of reads',
            'min': 0,
            'max': 15,
            'format': '{:,.1f}',
            'hidden': True,
            'scale': 'Greens',
        }
        headers['median.q'] = {
            'title': 'Median Q score',
            'description': 'Median quality of reads',
            'min': 0,
            'max': 15,
            'format': '{:,.1f}',
            'scale': 'Greens',
        }
        headers['mean.length'] = {
            'title': 'Mean length (bp)',
            'description': 'Mean read length',
            'format': '{:,.0f}',
            'hidden': True,
            'scale': 'Blues',
        }
        headers['median.length'] = {
            'title': 'Median length (bp)',
            'description': 'Median read length',
            'format': '{:,.0f}',
            'scale': 'Blues',
        }

        # Add row ID to avoid duplicates
        for k in headers:
            h_id = re.sub('[^0-9a-zA-Z]+', '_', headers[k]['title'])
            headers[k]['rid'] = "rid_{}".format(h_id)

        return headers


    def table_qALL(self):
        """ Table showing stats for all reads """

        self.add_section (
            name = 'Stats: All reads',
            anchor = 'minionqc-stats-qAll',
            description = 'MinIONQC statistics for all reads',
            plot = table.plot(
                self.minionqc_data,
                self.headers_to_use(),
                {
                    'namespace': 'MinIONQC',
                    'id': 'minionqc-stats-qAll-table',
                    'table_title': 'MinIONQC Stats: All reads'
                }
            )
        )


    def table_qfiltered(self):
        """ Table showing stats for q-filtered reads """

        description = 'MinIONQC statistics for quality filtered reads. ' + \
                        'Quailty threshold used: {}.'.format(', '.join(list(self.q_threshold_list)))
        if len(self.q_threshold_list) > 1:
            description += '''
            <div class="alert alert-warning">
              <span class="glyphicon glyphicon-warning-sign"></span>
              <strong>Warning!</strong> More than one quality thresholds were present.
            </div>
            '''
            log.warning('More than one quality thresholds were present. Thresholds: {}.'.format(', '.join(list(self.q_threshold_list))))

        self.add_section (
            name = 'Stats: Quality filtered reads',
            anchor = 'minionqc-stats-qFilt',
            description = description,
            plot = table.plot(
                self.qfilt_data,
                self.headers_to_use(),
                {
                    'namespace': 'MinIONQC',
                    'id': 'minionqc-stats-qFilt-table',
                    'table_title': 'MinIONQC Stats: Quality filtered reads'
                }
            )
        )

    def plot_readlengths(self):
        pdata = [
            { s_name: d['All reads']['reads'] for s_name,d in self.minionqc_raw_data.items() },
            { s_name: d['All reads']['gigabases'] for s_name,d in self.minionqc_raw_data.items() }
        ]
        pconfig = {
            'id': 'minionqc_read_lengths',
            'title': 'MinIONQC: Output versus read length',
            'categories': True,
            'ylab': '# reads',
            'data_labels': [
                {'name': 'All reads: Num reads', 'ylab': '# reads'},
                {'name': 'All reads: Num gigabases', 'ylab': '# gigabases'}
            ]
        }
        for qfilt in list(self.q_threshold_list):
            try:
                pdata.extend([
                    { s_name: d[qfilt]['reads'] for s_name,d in self.minionqc_raw_data.items() },
                    { s_name: d[qfilt]['gigabases'] for s_name,d in self.minionqc_raw_data.items() },
                ])
                pconfig['data_labels'].extend([
                    {'name': '{}: Num reads'.format(qfilt), 'ylab': '# reads'},
                    {'name': '{}: Num gigabases'.format(qfilt), 'ylab': '# gigabases'},
                ])
            except KeyError:
                pass
        self.add_section (
            name = 'Read length output',
            anchor = 'minionqc-read-length-output',
            description = 'Number of reads / bp sequenced at given read length thresholds.',
            plot = linegraph.plot(pdata, pconfig=pconfig)
        )



from collections import OrderedDict
from multiqc import config


read_format = '{:,.1f}'
if config.read_count_multiplier == 1:
    read_format = '{:,.0f}'
# read_format += '&nbsp;' + config.read_count_prefix

base_format = '{:,.1f}&nbsp;'
if config.base_count_multiplier == 1:
    base_format = '{:,.0f}'
elif config.base_count_multiplier == 0.000000001:
    base_format = '{:,.2f}'
# base_format += '&nbsp;' + config.base_count_prefix


class Metric:
    def __init__(self, id, title, in_genstats=None, in_own_tabl=None, unit=None,
                 descr=None, fmt=None, modify=None, namespace=None, precision=None,
                 the_higher_the_worse=False):
        self.id = id
        self.title = title
        self.in_genstats = in_genstats
        self.in_own_tabl = in_own_tabl
        self.unit = unit
        self.descr = descr
        self.fmt = fmt
        self.modify = modify
        self.namespace = namespace
        self.precision = precision
        self.the_higher_the_worse = the_higher_the_worse


def make_headers(parsed_metric_ids, metrics):
    # Init general stats table
    genstats_headers = OrderedDict()

    # Init headers for an own separate table
    own_tabl_headers = OrderedDict()

    for metric in metrics:
        col = dict(
            title=metric.title,
            description=metric.descr,
            min=0,
        )

        # choosing color based on metric namespace, guessing namespace by unit
        if metric.unit == 'reads':
            col['scale'] = 'Greens'
        elif metric.unit == 'bases':
            col['scale'] = 'Greys'
        elif metric.unit == 'bp':
            col['scale'] = 'Purples'
        elif metric.unit == 'x' or metric.namespace == 'Coverage':
            col['scale'] = 'Blues'
            if metric.unit == '%':
                col['scale'] = 'RdBu'
                if metric.the_higher_the_worse:
                    col['scale'] = 'YlOrRd'
        elif metric.namespace == 'Variants':
            col['scale'] = 'Purples'

        if metric.id + ' pct' in parsed_metric_ids:
            # if % value is available, showing it instead of the number value; the number value will be hidden
            pct_col = dict(
                col,
                description=metric.descr.replace(', {}', '').replace('Number of ', '% of '),
                max=100,
                suffix='%',
            )
            if metric.unit == 'reads':
                pct_col['scale'] = 'RdYlGn'
            elif metric.unit == 'bases':
                pct_col['scale'] = 'RdGy'
            elif metric.unit == 'x' or metric.namespace == 'Coverage':
                pct_col['scale'] = 'RdBu'
            if metric.the_higher_the_worse:
                pct_col['scale'] = 'YlOrRd'

            if metric.precision is not None:
                pct_col['format'] = '{:,.' + str(metric.precision) + 'f}'

            if metric.in_own_tabl is not None:
                show = metric.in_own_tabl == '%'
                own_tabl_headers[metric.id + ' pct'] = dict(pct_col, hidden=not show)
            if metric.in_genstats is not None and metric.in_own_tabl != '#':
                genstats_headers[metric.id + ' pct'] = dict(pct_col, hidden=metric.in_genstats == 'hid')

        if metric.unit == 'reads':
            col['description'] = col['description'].format(config.read_count_desc)
            col['title'] = config.read_count_prefix + ' ' + col['title']
            col['modify'] = lambda x: x * config.read_count_multiplier
            col['shared_key'] = 'read_count'
            col['format'] = read_format
        elif metric.unit == 'bases':
            col['description'] = col['description'].format(config.base_count_desc)
            col['title'] = config.base_count_prefix + ' ' + col['title']
            col['modify'] = lambda x: x * config.base_count_multiplier
            col['shared_key'] = 'base_count'
            col['format'] = base_format
        elif metric.unit == 'len':
            col['suffix'] = ' bp'
            col['format'] = '{:,.0f}'
        elif metric.unit == 'x':
            col['suffix'] = ' x'
            col['format'] = '{:,.1f}'
        elif metric.unit == '%':
            col['suffix'] = ' %'
            col['format'] = '{:,.1f}'
        elif any(metric.descr.startswith(pref) for pref in ('Total number of ', 'The number of ', 'Number of ')):
            col['format'] = '{:,.0f}'
        if metric.precision is not None:
            col['format'] = '{:,.' + str(metric.precision) + 'f}'

        if metric.in_own_tabl is not None:
            own_tabl_headers[metric.id] = dict(col, hidden=metric.in_own_tabl != '#')
        if metric.in_genstats is not None:
            if metric.in_genstats == '#':
                genstats_headers[metric.id] = dict(col)
            elif metric.in_genstats == 'hid' and metric.in_own_tabl == '#':
                genstats_headers[metric.id] = dict(col, hidden=metric.in_genstats == 'hid')

    return genstats_headers, own_tabl_headers



#!/usr/bin/env python

""" MultiQC module to parse TsTv by quality output from vcftools TsTv-by-qual """

import logging
from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)

class TsTvByQualMixin():

    def parse_tstv_by_qual(self):
        """ Create the HTML for the TsTv by quality linegraph plot. """

        self.vcftools_tstv_by_qual = dict()
        for f in self.find_log_files('vcftools/tstv_by_qual', filehandles=True):
            d = {}
            for line in f['f'].readlines()[1:]: # don't add the header line (first row)
                key = float(line.split()[0]) # taking the first column (QUAL_THRESHOLD) as key
                val = float(line.split()[6]) # taking Ts/Tv_GT_QUAL_THRESHOLD as value
                if (val == float('inf')) or (val == float('-inf')):
                    val = float('nan')
                d[key] = val
            self.vcftools_tstv_by_qual[f['s_name']] = d

        # Filter out ignored sample names
        self.vcftools_tstv_by_qual = self.ignore_samples(self.vcftools_tstv_by_qual)

        if len(self.vcftools_tstv_by_qual) == 0:
            return 0

        pconfig = {
            'id': 'vcftools_tstv_by_qual',
            'title': 'VCFTools: TsTv by Qual',
            'ylab': 'TsTv Ratio',
            'xlab': 'SNP Quality Threshold',
            'xmin': 0,
            'ymin': 0,
            'smooth_points': 400, # this limits huge filesizes and prevents browser crashing
            'smooth_points_sumcounts': False
        }

        helptext = '''
        `Transition` is a purine-to-purine or pyrimidine-to-pyrimidine point mutations.
        `Transversion` is a purine-to-pyrimidine or pyrimidine-to-purine point mutation.
        `Quality` here is the Phred-scaled quality score as given in the QUAL column of VCF.
        Note: only bi-allelic SNPs are used (multi-allelic sites and INDELs are skipped.)
        Refer to Vcftools's manual (https://vcftools.github.io/man_latest.html) on `--TsTv-by-qual`
        '''

        self.add_section(
            name = 'TsTv by Qual',
            anchor = 'vcftools-tstv-by-qual',
            description = "Plot of `TSTV-BY-QUAL` - the transition to transversion ratio as a function of SNP quality from the output of vcftools TsTv-by-qual.",
            helptext = helptext,
            plot = linegraph.plot(self.vcftools_tstv_by_qual,pconfig)
        )

        return len(self.vcftools_tstv_by_qual)




#!/usr/bin/env python

""" MultiQC module to parse output from Slamdunk """

from __future__ import print_function
import logging
import re
from collections import OrderedDict

from multiqc import config
from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.plots import table, bargraph, linegraph, scatter

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """
    Slamdunk module class, parses slamdunk logs.
    """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Slamdunk', anchor='slamdunk',
        href='http://t-neumann.github.io/slamdunk/',
        info="is a tool to analyze SLAMSeq data.")

        num_reports = 0

        self.plot_cols = [
            '#fdbf6f','#2171B5','#6BAED6','#C6DBEF','#74C476','#C7E9C0',
            '#D9D9D9','#969696','#525252','#DADAEB','#9E9AC8','#6A51A3'
        ]

        # Summary Reports
        self.slamdunk_data = dict()
        for f in self.find_log_files('slamdunk/summary', filehandles = True):
            self.parseSummary(f)
        self.slamdunk_data = self.ignore_samples(self.slamdunk_data)
        if len(self.slamdunk_data) > 0:
            self.slamdunkGeneralStatsTable()
            self.slamdunkFilterStatsTable()
            log.debug("Found {} summary reports".format(len(self.slamdunk_data)))
            num_reports = max(num_reports, len(self.slamdunk_data))

        # PCA Plots
        self.PCA_data = dict()
        for f in self.find_log_files('slamdunk/PCA', filehandles = True):
            self.parsePCA(f)
        self.PCA_data = self.ignore_samples(self.PCA_data)
        if len(self.PCA_data) > 0:
            self.slamdunkPCAPlot()
            log.debug("Found {} PCA plots".format(len(self.PCA_data)))
            num_reports = max(num_reports, len(self.PCA_data))

        # UTR Rate reports
        self.utrates_data = dict()
        for f in self.find_log_files('slamdunk/utrrates', filehandles = True):
            self.parseUtrRates(f)
        self.utrates_data = self.ignore_samples(self.utrates_data)
        if len(self.utrates_data) > 0:
            self.write_data_file(self.utrates_data, 'multiqc_slamdunk_utrrates')
            self.slamdunkUtrRatesPlot()
            log.debug("Found {} UTR rate reports".format(len(self.utrates_data)))
            num_reports = max(num_reports, len(self.utrates_data))

        # Read rate reports
        self.rates_data_plus = dict()
        self.rates_data_minus = dict()
        for f in self.find_log_files('slamdunk/rates', filehandles = True):
            self.parseSlamdunkRates(f)
        self.rates_data_plus = self.ignore_samples(self.rates_data_plus)
        self.rates_data_minus = self.ignore_samples(self.rates_data_minus)
        if len(self.rates_data_plus) > 0:
            self.write_data_file(self.rates_data_plus, 'multiqc_slamdunk_readrates_plus')
            self.write_data_file(self.rates_data_minus, 'multiqc_slamdunk_readrates_minus')
            self.slamdunkOverallRatesPlot()
            log.debug("Found {} read rate reports".format(len(self.rates_data_plus)))
            num_reports = max(num_reports, len(self.rates_data_plus))

        # TCP error read rate
        self.nontc_per_readpos_plus = dict()
        self.nontc_per_readpos_minus = dict()
        self.tc_per_readpos_plus = dict()
        self.tc_per_readpos_minus = dict()
        for f in self.find_log_files('slamdunk/tcperreadpos', filehandles = True):
            self.parseSlamdunkTCPerReadpos(f)
        self.nontc_per_readpos_plus = self.ignore_samples(self.nontc_per_readpos_plus)
        self.nontc_per_readpos_minus = self.ignore_samples(self.nontc_per_readpos_minus)
        self.tc_per_readpos_plus = self.ignore_samples(self.tc_per_readpos_plus)
        self.tc_per_readpos_minus = self.ignore_samples(self.tc_per_readpos_minus)
        if len(self.tc_per_readpos_plus) > 0:
            self.write_data_file(self.tc_per_readpos_plus, 'multiqc_slamdunk_tcperreadpos_plus')
            self.write_data_file(self.nontc_per_readpos_plus, 'multiqc_slamdunk_nontcperreadpos_plus')
            self.write_data_file(self.tc_per_readpos_minus, 'multiqc_slamdunk_tcperreadpos_minus')
            self.write_data_file(self.nontc_per_readpos_minus, 'multiqc_slamdunk_nontcperreadpos_minus')
            self.slamdunkTcPerReadPosPlot()
            log.debug("Found {} TCP error read rate reports".format(len(self.tc_per_readpos_plus)))
            num_reports = max(num_reports, len(self.tc_per_readpos_plus))

        # Non-TCP error read rate
        self.nontc_per_utrpos_plus = dict()
        self.nontc_per_utrpos_minus = dict()
        self.tc_per_utrpos_plus = dict()
        self.tc_per_utrpos_minus = dict()
        for f in self.find_log_files('slamdunk/tcperutrpos', filehandles = True):
            self.parseSlamdunkTCPerUtrpos(f)
        self.nontc_per_utrpos_plus = self.ignore_samples(self.nontc_per_utrpos_plus)
        self.nontc_per_utrpos_minus = self.ignore_samples(self.nontc_per_utrpos_minus)
        self.tc_per_utrpos_plus = self.ignore_samples(self.tc_per_utrpos_plus)
        self.tc_per_utrpos_minus = self.ignore_samples(self.tc_per_utrpos_minus)
        if len(self.nontc_per_utrpos_plus) > 0:
            self.write_data_file(self.tc_per_utrpos_plus, 'multiqc_slamdunk_tcperutrpos_plus')
            self.write_data_file(self.nontc_per_utrpos_plus, 'multiqc_slamdunk_nontcperutrpos_plus')
            self.write_data_file(self.tc_per_utrpos_minus, 'multiqc_slamdunk_tcperutrpos_minus')
            self.write_data_file(self.nontc_per_utrpos_minus, 'multiqc_slamdunk_nontcperutrpos_minus')
            self.slamdunkTcPerUTRPosPlot()
            log.debug("Found {} non TCP error read rate reports".format(len(self.nontc_per_utrpos_plus)))
            num_reports = max(num_reports, len(self.nontc_per_utrpos_plus))

        if num_reports == 0:
            raise UserWarning
        else:
            log.info("Found {} reports".format(num_reports))


    def parsePCA(self, f):

        # Skip header
        next(f['f'])

        for line in f['f']:
            fields = line.rstrip().split('\t')

            sample = self.clean_s_name(fields[0],f['root'])
            PC1 = fields[1]
            PC2 = fields[2]

            self.PCA_data[sample] = [{'x': float(PC1), 'y': float(PC2)}]


    def parseUtrRates(self, f):

        # Skip comment line #
        next(f['f'])

        # Read median header
        line = next(f['f'])

        if "Conversions=" in line:

            sample = f['s_name']
            self.utrates_data[sample] = OrderedDict()

            conversions = re.sub(".*Conversions=","",line.rstrip()).split(",")

            for conversion in conversions:
                type, value = conversion.split(":")
                self.utrates_data[sample][type] = float(value)

        else:
            log.warning("Malformed UTR rates header. Conversion rates per UTR plot will be affected.")

    def parseSlamdunkRates(self, f):

        sample = f['s_name']

        # Skip comment line #
        next(f['f'])

        bases = next(f['f']).rstrip().split('\t')

        baseDict = {}
        order = {}

        for i in range(1, len(bases)):
            order[i] = bases[i]

        for line in f['f']:
            values = line.rstrip().split('\t')
            base = values[0]
            baseDict[base]= {}

            for i in range(1, len(values)):
                baseDict[base][order[i]] = int(values[i])

        divisor = {}

        for fromBase in baseDict:
            for toBase in baseDict[fromBase]:
                if(toBase.islower()):
                    if not fromBase.lower() in divisor:
                        divisor[fromBase.lower()] = 0
                    divisor[fromBase.lower()] += baseDict[fromBase][toBase]
                else:
                    if not fromBase in divisor:
                        divisor[fromBase] = 0
                    divisor[fromBase] += baseDict[fromBase][toBase]

        for fromBase in baseDict:
            for toBase in baseDict[fromBase]:
                if(toBase.islower()):
                    if divisor[fromBase.lower()] > 0:
                        baseDict[fromBase][toBase] = baseDict[fromBase][toBase] / float(divisor[fromBase.lower()]) * 100
                    else :
                        baseDict[fromBase][toBase] = 0.0
                else:
                    if divisor[fromBase] > 0:
                        baseDict[fromBase][toBase] = baseDict[fromBase][toBase] / float(divisor[fromBase]) * 100
                    else :
                        baseDict[fromBase][toBase] = 0.0

        self.rates_data_plus[sample] = {}
        self.rates_data_minus[sample] = {}

        for fromBase in baseDict:
            for toBase in baseDict[fromBase]:
                if fromBase != "N" and toBase.upper() != "N" and fromBase != toBase.upper():
                    if(toBase.islower()):
                        self.rates_data_minus[sample][fromBase + ">" + toBase.upper()] = baseDict[fromBase][toBase]
                    else:
                        self.rates_data_plus[sample][fromBase + ">" + toBase] = baseDict[fromBase][toBase]

    def parseSlamdunkTCPerReadpos(self, f):

        sample = f['s_name']

        # Skip comment line #
        next(f['f'])

        self.nontc_per_readpos_plus[sample] = {}
        self.nontc_per_readpos_minus[sample] = {}

        self.tc_per_readpos_plus[sample] = {}
        self.tc_per_readpos_minus[sample] = {}

        pos = 1

        for line in f['f']:
            values = line.rstrip().split('\t')
            if int(values[4]) > 0:
                self.nontc_per_readpos_plus[sample][pos] = float(int(values[0])) / int(values[4]) * 100
                self.tc_per_readpos_plus[sample][pos] = float(int(values[2])) / int(values[4]) * 100
            else:
                self.nontc_per_readpos_plus[sample][pos] = 0
                self.tc_per_readpos_plus[sample][pos] = 0

            if int(values[5]) > 0:
                self.nontc_per_readpos_minus[sample][pos] = float(int(values[1])) / int(values[5]) * 100
                self.tc_per_readpos_minus[sample][pos] = float(int(values[3])) / int(values[5]) * 100
            else:
                self.nontc_per_readpos_minus[sample][pos] = 0
                self.tc_per_readpos_minus[sample][pos] = 0

            pos += 1

    def parseSlamdunkTCPerUtrpos(self, f):

        sample = f['s_name']

        # Skip comment line #
        next(f['f'])

        self.nontc_per_utrpos_plus[sample] = {}
        self.nontc_per_utrpos_minus[sample] = {}

        self.tc_per_utrpos_plus[sample] = {}
        self.tc_per_utrpos_minus[sample] = {}

        pos = 1

        for line in f['f']:
            values = line.rstrip().split('\t')
            if int(values[4]) > 0:
                self.nontc_per_utrpos_plus[sample][pos] = float(int(values[0])) / int(values[4]) * 100
                self.tc_per_utrpos_plus[sample][pos] = float(int(values[2])) / int(values[4]) * 100
            else:
                self.nontc_per_utrpos_plus[sample][pos] = 0
                self.tc_per_utrpos_plus[sample][pos] = 0

            if int(values[5]) > 0:
                self.nontc_per_utrpos_minus[sample][pos] = float(int(values[1])) / int(values[5]) * 100
                self.tc_per_utrpos_minus[sample][pos] = float(int(values[3])) / int(values[5]) * 100
            else:
                self.nontc_per_utrpos_minus[sample][pos] = 0
                self.tc_per_utrpos_minus[sample][pos] = 0

            pos += 1

    def parseSummary(self, f):

        # Skip comment line #
        next(f['f'])

        # Skip header line "FileName..."
        columnCount = next(f['f']).count("\t") + 1

        for line in f['f']:

            fields = line.rstrip().split("\t")
            s_name = self.clean_s_name(fields[0],f['root'])
            self.slamdunk_data[s_name] = dict()
            self.slamdunk_data[s_name]['sequenced'] = int(fields[4])
            self.slamdunk_data[s_name]['mapped'] = int(fields[5])
            #self.slamdunk_data[s_name]['deduplicated'] = int(fields[6])
            self.slamdunk_data[s_name]['mqfiltered'] = int(fields[7])
            self.slamdunk_data[s_name]['idfiltered'] = int(fields[8])
            self.slamdunk_data[s_name]['nmfiltered'] = int(fields[9])
            self.slamdunk_data[s_name]['multimapper'] = int(fields[10])
            self.slamdunk_data[s_name]['retained'] = int(fields[11])

            # Additional Count Column found in Table
            if columnCount == 14:
                self.slamdunk_data[s_name]['counted'] = int(fields[12])

        self.add_data_source(f)

    def slamdunkGeneralStatsTable(self):
        """ Take the parsed summary stats from Slamdunk and add it to the
        basic stats table at the top of the report """


        headers = OrderedDict()
        headers['counted'] = {
            'title': '{} Counted'.format(config.read_count_prefix),
            'description': '# reads counted within 3\'UTRs ({})'.format(config.read_count_desc),
            'shared_key': 'read_count',
            'min': 0,
            'format': '{:,.2f}',
            'scale': 'YlGn',
            'modify': lambda x: float(x) * config.read_count_multiplier,
        }
        headers['retained'] = {
            'title': '{} Retained'.format(config.read_count_prefix),
            'description': '# retained reads after filtering ({})'.format(config.read_count_desc),
            'shared_key': 'read_count',
            'min': 0,
            'format': '{:,.2f}',
            'scale': 'YlGn',
            'modify': lambda x: float(x) * config.read_count_multiplier,
        }
        headers['mapped'] = {
            'title': '{} Mapped'.format(config.read_count_prefix),
            'description': '# mapped reads ({})'.format(config.read_count_desc),
            'shared_key': 'read_count',
            'min': 0,
            'format': '{:,.2f}',
            'scale': 'YlGn',
            'modify': lambda x: float(x) * config.read_count_multiplier,
        }
        headers['sequenced'] = {
            'title': '{} Sequenced'.format(config.read_count_prefix),
            'description': '# sequenced reads ({})'.format(config.read_count_desc),
            'shared_key': 'read_count',
            'min': 0,
            'format': '{:,.2f}',
            'scale': 'YlGn',
            'modify': lambda x: float(x) * config.read_count_multiplier,
        }

        self.general_stats_addcols(self.slamdunk_data, headers)

    def slamdunkFilterStatsTable(self):
        """ Take the parsed filter stats from Slamdunk and add it to a separate table """

        headers = OrderedDict()
        headers['mapped'] = {
            'namespace': 'Slamdunk',
            'title': '{} Mapped'.format(config.read_count_prefix),
            'description': '# mapped reads ({})'.format(config.read_count_desc),
            'shared_key': 'read_count',
            'min': 0,
            'format': '{:,.2f}',
            'suffix': config.read_count_prefix,
            'scale': 'YlGn',
            'modify': lambda x: float(x) * config.read_count_multiplier,
        }
        headers['multimapper'] = {
            'namespace': 'Slamdunk',
            'title': '{} Multimap-Filtered'.format(config.read_count_prefix),
            'description': '# multimap-filtered reads ({})'.format(config.read_count_desc),
            'shared_key': 'read_count',
            'min': 0,
            'format': '{:,.2f}',
            'suffix': config.read_count_prefix,
            'scale': 'OrRd',
            'modify': lambda x: float(x) * config.read_count_multiplier,
        }
        headers['nmfiltered'] = {
            'namespace': 'Slamdunk',
            'title': '{} NM-Filtered'.format(config.read_count_prefix),
            'description': '# NM-filtered reads ({})'.format(config.read_count_desc),
            'shared_key': 'read_count',
            'min': 0,
            'format': '{:,.2f}',
            'suffix': config.read_count_prefix,
            'scale': 'OrRd',
            'modify': lambda x: float(x) * config.read_count_multiplier,
        }
        headers['idfiltered'] = {
            'namespace': 'Slamdunk',
            'title': '{} Identity-Filtered'.format(config.read_count_prefix),
            'description': '# identity-filtered reads ({})'.format(config.read_count_desc),
            'shared_key': 'read_count',
            'min': 0,
            'format': '{:,.2f}',
            'suffix': config.read_count_prefix,
            'scale': 'OrRd',
            'modify': lambda x: float(x) * config.read_count_multiplier,
        }
        headers['mqfiltered'] = {
            'namespace': 'Slamdunk',
            'title': '{} MQ-Filtered'.format(config.read_count_prefix),
            'description': '# MQ-filtered reads ({})'.format(config.read_count_desc),
            'shared_key': 'read_count',
            'min': 0,
            'format': '{:,.2f}',
            'suffix': config.read_count_prefix,
            'scale': 'OrRd',
            'modify': lambda x: float(x) * config.read_count_multiplier,
        }
        pconfig = {
            'id': 'slamdunk_filtering_table',
            'min': 0,
        }

        self.add_section (
            name = 'Filter statistics',
            anchor = 'slamdunk_filtering',
            description = 'This table shows the number of reads filtered with each filter criterion during filtering phase of slamdunk.',
            plot = table.plot(self.slamdunk_data, headers, pconfig)
        )

    def slamdunkOverallRatesPlot (self):
        """ Generate the overall rates plot """

        pconfig = {
            'id': 'overallratesplot',
            'title': 'Slamdunk: Overall conversion rates in reads',
            'cpswitch': False,
            'cpswitch_c_active': False,
            'ylab': 'Number of reads',
            'stacking': 'normal',
            'tt_decimals': 2,
            'tt_suffix': '%',
            'tt_percentages': False,
            'hide_zero_cats': False,
            'data_labels': [
                "Plus Strand +",
                "Minus Strand -",
            ]
        }

        cats = [OrderedDict(), OrderedDict()]
        keys = [
            ['T>C', 'A>T', 'A>G', 'A>C', 'T>A', 'T>G', 'G>A', 'G>T', 'G>C', 'C>A', 'C>T', 'C>G'],
            ['A>G','A>T','A>C','T>A','T>G','T>C','G>A','G>T','G>C','C>A','C>T','C>G']
        ]
        for i, k in enumerate(keys):
            for j, v in enumerate(k):
                cats[i][v] = { 'color': self.plot_cols[j] }

        self.add_section (
            name = 'Conversion rates per read',
            anchor = 'slamdunk_overall_rates',
            description = """This plot shows the individual conversion rates over all reads.
                        It shows these conversion rates strand-specific: This means for a properly labeled
                        sample you would see a T&gt;C excess on the plus-strand and an A&gt;G excess on the minus strand
                        (see the <a href="http://t-neumann.github.io/slamdunk/docs.html#rates" target="_blank">slamdunk docs</a>).""",
            plot = bargraph.plot([self.rates_data_plus,self.rates_data_minus], cats, pconfig)
        )


    def slamdunkUtrRatesPlot (self):
        """ Generate the UTR rates plot """

        cats = OrderedDict()
        keys = ['T>C', 'A>T', 'A>G', 'A>C', 'T>A', 'T>G', 'G>A', 'G>T', 'G>C', 'C>A', 'C>T', 'C>G']
        for i, v in enumerate(keys):
            cats[v] = { 'color': self.plot_cols[i] }

        pconfig = {
            'id': 'slamdunk_utrratesplot',
            'title': 'Slamdunk: Overall conversion rates per UTR',
            'cpswitch': False,
            'cpswitch_c_active': False,
            'ylab': 'Number of conversions',
            'stacking': 'normal',
            'tt_decimals': 2,
            'tt_suffix': '%',
            'tt_percentages': False,
            'hide_zero_cats': False
        }

        self.add_section (
            name = 'Conversion rates per UTR',
            anchor = 'slamdunk_utr_rates',
            description = """This plot shows the individual conversion rates for all UTRs
                        (see the <a href="http://t-neumann.github.io/slamdunk/docs.html#utrrates" target="_blank">slamdunk docs</a>).""",
            plot = bargraph.plot(self.utrates_data, cats, pconfig)
        )

    def slamdunkPCAPlot (self):
        """ Generate the PCA plots """

        pconfig = {
            'id': 'slamdunk_pca',
            'title': 'Slamdunk: PCA',
            'xlab': 'PC1',
            'ylab': 'PC2',
            'tt_label': 'PC1 {point.x:.2f}: PC2 {point.y:.2f}'
        }

        self.add_section (
            name = 'PCA (T&gt;C based)',
            anchor = 'slamdunk_PCA',
            description = """This plot shows the principal components of samples based
                        on the distribution of reads with T&gt;C conversions within UTRs
                        (see the <a href="http://t-neumann.github.io/slamdunk/docs.html#summary" target="_blank">slamdunk docs</a>).""",
            plot = scatter.plot(self.PCA_data, pconfig)
        )

    def slamdunkTcPerReadPosPlot (self):
        """ Generate the tc per read pos plots """

        pconfig_nontc = {
            'id': 'slamdunk_nontcperreadpos_plot',
            'title': 'Slamdunk: Non-T>C mismatches over reads',
            'ylab': 'Percent mismatches %',
            'xlab': 'Position in read',
            'xDecimals': False,
            'ymin': 0,
            'tt_label': '<b>Pos {point.x}</b>: {point.y:.2f} %',
            'data_labels': [{'name': 'Forward reads +', 'ylab': 'Percent mismatches %'},
                            {'name': 'Reverse reads -', 'ylab': 'Percent mismatches %'}]
        }

        pconfig_tc = {
            'id': 'slamdunk_tcperreadpos_plot',
            'title': 'Slamdunk: T>C conversions over reads',
            'ylab': 'Percent converted %',
            'xlab': 'Position in read',
            'xDecimals': False,
            'ymin': 0,
            'tt_label': '<b>Pos {point.x}</b>: {point.y:.2f} %',
            'data_labels': [{'name': 'Forward reads +', 'ylab': 'Percent converted %'},
                            {'name': 'Reverse reads -', 'ylab': 'Percent converted %'}]
        }

        self.add_section (
            name = 'Non T&gt;C mismatches over read positions',
            anchor = 'slamdunk_nontcperreadpos',
            description = """This plot shows the distribution of non T&gt;C mismatches across read positions
                        (see the <a href="http://t-neumann.github.io/slamdunk/docs.html#tcperreadpos" target="_blank">slamdunk docs</a>).""",
            plot = linegraph.plot([self.nontc_per_readpos_plus, self.nontc_per_readpos_minus], pconfig_nontc)
        )

        self.add_section (
            name = 'T&gt;C conversions over read positions',
            anchor = 'slamdunk_tcperreadpos',
            description = """This plot shows the distribution of T&gt;C conversions across read positions
                        (see the <a href="http://t-neumann.github.io/slamdunk/docs.html#tcperreadpos" target="_blank">slamdunk docs</a>).""",
            plot = linegraph.plot([self.tc_per_readpos_plus, self.tc_per_readpos_minus], pconfig_tc)
        )

    def slamdunkTcPerUTRPosPlot (self):
        """ Generate the tc per UTR pos plots """

        pconfig_nontc = {
            'id': 'slamdunk_slamdunk_nontcperutrpos_plot',
            'title': 'Slamdunk: Non-T>C mutations over 3\' UTR ends',
            'ylab': 'Percent mismatches %',
            'xlab': 'Position in the static last 250bp window of 3\' UTR',
            'xDecimals': False,
            'ymin': 0,
            'tt_label': '<b>Pos {point.x}</b>: {point.y:.2f} %',
            'data_labels': [{'name': 'UTRs on plus strand', 'ylab': 'Percent mismatches %'},
                            {'name': 'UTRs on minus strand', 'ylab': 'Percent mismatches %'}]
        }

        pconfig_tc = {
            'id': 'slamdunk_slamdunk_tcperutrpos_plot',
            'title': 'Slamdunk: T>C conversions over 3\' UTR ends',
            'ylab': 'Percent converted %',
            'xlab': 'Position in the static last 250bp window of 3\' UTR',
            'xDecimals': False,
            'ymin': 0,
            'tt_label': '<b>Pos {point.x}</b>: {point.y:.2f} %',
            'data_labels': [{'name': 'UTRs on plus strand', 'ylab': 'Percent converted %'},
                            {'name': 'UTRs on minus strand', 'ylab': 'Percent converted %'}]
        }

        self.add_section (
            name = 'Non T&gt;C mismatches over UTR positions',
            anchor = 'slamdunk_nontcperutrpos',
            description = """This plot shows the distribution of non T&gt;C mismatches across UTR positions for the last 250 bp from the 3\' UTR end
                        (see the <a href="http://t-neumann.github.io/slamdunk/docs.html#tcperutrpos" target="_blank">slamdunk docs</a>).""",
            plot = linegraph.plot([self.nontc_per_utrpos_plus, self.nontc_per_utrpos_minus], pconfig_nontc)
        )

        self.add_section (
            name = 'T&gt;C conversions over UTR positions',
            anchor = 'tcperutrpos',
            description = """This plot shows the distribution of T&gt;C conversions across UTR positions for the last 250 bp from the 3\' UTR end
                        (see the <a href="http://t-neumann.github.io/slamdunk/docs.html#tcperutrpos" target="_blank">slamdunk docs</a>).""",
            plot = linegraph.plot([self.tc_per_utrpos_plus, self.tc_per_utrpos_minus], pconfig_tc)
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" MultiQC module to parse output files from VarScan2 """

from __future__ import print_function
from collections import OrderedDict
import logging
import re

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name = 'VarScan2',
            anchor = 'varscan',
            href = 'http://dkoboldt.github.io/varscan/',
            info = "variant detection in massively parallel sequencing data"
        )

        # Find and load VarScan2 reports - there are 3 different ones, but all with identical content (differentiated by header)
        self.varscan2_data = dict()
        for f in self.find_log_files('varscan2/mpileup2snp', filehandles=True):
            parsed_data = self.parse_varscan(f)
            s_name = self.clean_s_name(parsed_data['sample_name'], f['root'])
            # Drop existing sample_name from dict now
            del parsed_data['sample_name']
            if parsed_data is not None and len(parsed_data) > 0:
                self.varscan2_data[s_name] = parsed_data
                self.add_data_source(f, s_name, section="mpileup2snp")

        for f in self.find_log_files('varscan2/mpileup2indel', filehandles=True):
            parsed_data = self.parse_varscan(f)
            s_name = self.clean_s_name(parsed_data['sample_name'], f['root'])
            # Drop existing sample_name from dict now
            del parsed_data['sample_name']
            if parsed_data is not None and len(parsed_data) > 0:
                self.varscan2_data[s_name] = parsed_data
                self.add_data_source(f, s_name, section="mpileup2indel")

        for f in self.find_log_files('varscan2/mpileup2cns', filehandles=True):
            parsed_data = self.parse_varscan(f)
            s_name = self.clean_s_name(parsed_data['sample_name'], f['root'])
            # Drop existing sample_name from dict now
            del parsed_data['sample_name']
            if parsed_data is not None and len(parsed_data) > 0:
                self.varscan2_data[s_name] = parsed_data
                self.add_data_source(f, s_name, section="mpileup2cns")

        # Filter to strip out ignored sample names
        self.varscan2_data = self.ignore_samples(self.varscan2_data)

        # Warning when no files are found
        if len(self.varscan2_data) == 0:
            raise UserWarning

        # Write parsed data to a file
        self.write_data_file(self.varscan2_data, 'multiqc_varscan2_summary')

        # Found reports or not?
        log.info("Found {} reports".format(len(self.varscan2_data)))

        # Basic Stats Table
        self.varscan2_general_stats_table()

        # Basic barplot section
        self.varscan2_counts_barplot()

    # Varscan2 reports in SNP mode only snps, in indel only indel and in CNS all variants found
    # Total variants = SNPs + Indels
    def parse_varscan(self, f):
        """ Parse a VarScan2 report
        """
        parsed = dict()
        regexes = {
            'sample_name': r'(?:Reading input from )(\w+.+)',
            'min_coverage': r'(?:Min coverage:)\s(\d+)',
            'min_reads2': r'(?:Min reads2:)\s(\d+)',
            'min_var_freq': r'(?:Min var freq:)\s(\d+\.\d+)',
            'min_avg_qual': r'(?:Min avg qual:)\s(\d+)',
            'p_val_threshold': r'(?:P-value thresh:)\s(\d+\.\d+)',
            'bases_in_pileup': r'(\d+)(?:\sbases in pileup file)',
            'variant_total': r'(\d+)(?:\svariant positions \()',
            'variant_snps': r'(?:\d+)(?:\svariant positions \()(\d+)',
            'variant_indels': r'(?:\d+)(?:\svariant positions \()(?:\d+ SNP, )(\d+)',
            'failed_strand_filter': r'(\d+)(?:\swere failed by the strand-filter)',
            'variant_reported_total': r'(\d+)(?:\svariant positions reported)',
            'variant_reported_snps': r'(?:\d+)(?:\svariant positions reported \()(\d+)',
            'variant_reported_indels': r'(?:\d+)(?:\svariant positions reported \()(?:\d+ SNP, )(\d+)'
        }
        for l in f['f']:
            # Search regexes for stats
            for k, r in regexes.items():
                match = re.search(r, l)
                if match:
                    if k not in ['sample_name','p_val_threshold','min_var_freq']:
                        parsed[k] = int(match.group(1))
                    if k == 'sample_name':
                        parsed[k] = match.group(1)
                    if k in ['p_val_threshold','min_var_freq']:
                        parsed[k] = float(match.group(1))

        # The failed_strand_filter combines the different classes, so recompute
        if 'variant_total' in parsed and 'variant_reported_total' in parsed:
            parsed['variant_total_failed'] = parsed['variant_total'] - parsed['variant_reported_total']
        if 'variant_snps' in parsed and 'variant_reported_snps' in parsed:
            parsed['variant_snps_failed'] = parsed['variant_snps'] - parsed['variant_reported_snps']
        if 'variant_indels' in parsed and 'variant_reported_indels' in parsed:
            parsed['variant_indels_failed'] = parsed['variant_indels'] - parsed['variant_reported_indels']

        return parsed

    # Add to general stats table
    def varscan2_general_stats_table(self):
        """ Take the parsed stats from the VarScan2 report and add it to the
        General Statistics table
        """

        headers = OrderedDict()
        headers['variant_reported_snps'] = {
            'title': 'SNPs reported',
            'description': 'Total number of SNPs reported.',
            'min': 0,
            'scale': 'Spectral',
            'format': '{:,.0f}',
            'shared_key': 'snps'
        }
        headers['variant_reported_indels'] = {
            'title': 'INDELs reported',
            'description': 'Total number INDELs reported.',
            'min': 0,
            'scale': 'Spectral',
            'format': '{:,.0f}',
            'shared_key': 'indels'
        }
        headers['variant_reported_total'] = {
            'title': 'Variants reported',
            'description': 'Total number of variants reported.',
            'min': 0,
            'scale': 'Spectral',
            'format': '{:,.0f}',
            'shared_key': 'variants',
            'hidden': True
        }
        headers['variant_snps'] = {
            'title': 'SNPs',
            'description': 'Total number of SNPs detected',
            'min': 0,
            'scale': 'BrBg',
            'format': '{:,.0f}',
            'shared_key': 'snps',
            'hidden': True
        }
        headers['variant_indels'] = {
            'title': 'INDELs',
            'description': 'Total number of INDELs detected',
            'min': 0,
            'scale': 'BrBg',
            'format': '{:,.0f}',
            'shared_key': 'indels',
            'hidden': True
        }
        headers['variant_total'] = {
            'title': 'Variants',
            'description': 'Total number of variants detected',
            'min': 0,
            'scale': 'BrBg',
            'format': '{:,.0f}',
            'shared_key': 'variants',
            'hidden': True
        }
        headers['failed_strand_filter'] = {
            'title': 'Failed Strand Filter',
            'description': 'Total number variants failing the strand-filter.',
            'min': 0,
            'scale': 'YlOrBr',
            'format': '{:,.0f}',
            'shared_key': 'variants',
            'hidden': True
        }
        headers['bases_in_pileup'] = {
            'title': '{} Bases in Pileup'.format(config.base_count_prefix),
            'description': 'Number of bases in pileup input for VarScan2 ()'.format(config.base_count_desc),
            'scale': 'Greens',
            'modify': lambda x: x * config.base_count_multiplier,
            'shared_key': 'base_count',
            'hidden': True
        }

        self.general_stats_addcols(self.varscan2_data, headers)

    def varscan2_counts_barplot (self):
        """ Make the HighCharts HTML to plot the reported SNPs"""
        # 146 variant positions (106 SNP, 40 indel)
        # 12 were failed by the strand-filter
        # 99 variant positions reported (99 SNP, 0 indel)

        # Specify the order of the different possible categories
        cats = [ OrderedDict(), OrderedDict() ]
        cats[0]['variant_reported_snps'] = {'name': 'SNPs reported', 'color': '#7fc97f'}
        cats[0]['variant_snps_failed'] = {'name': 'SNPs not reported', 'color': '#fb8072'}
        cats[1]['variant_reported_indels'] = {'name': 'INDELs reported', 'color': '#7fc97f'}
        cats[1]['variant_indels_failed'] = {'name': 'INDELs not reported', 'color': '#fb8072'}

        # Config for the plot
        pconfig = {
            'id': 'varscan2_variant_counts_plot',
            'title': 'VarScan2: Variants detected',
            'ylab': 'Number of SNPs',
            'cpswitch_counts_label': 'Number of Variants',
            'hide_zero_cats': False,
            'data_labels': [
                {'name': 'SNPs', 'ylab': 'Number of SNPs'},
                {'name': 'INDELs', 'ylab': 'Number of INDELs'}
            ]
        }

        self.add_section(
            name = 'Variants detected',
            anchor = 'varscan2_variant_counts',
            description = '''
            This plot shows the total number of variant positions, broken down by those that were reported or not.
            ''',
            plot = bargraph.plot(
                [self.varscan2_data, self.varscan2_data],
                cats,
                pconfig
            )
        )



#!/usr/bin/env python

""" MultiQC module to parse output from Adapter Removal """

from __future__ import print_function
from collections import OrderedDict
import logging

from multiqc import config
from multiqc.plots import bargraph, linegraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Adapter Removal',
        anchor='adapterRemoval', target='Adapter Removal',
        href='https://github.com/MikkelSchubert/adapterremoval',
        info=" rapid adapter trimming, identification, and read merging ")

        self.__read_type = None
        self.__any_paired = False
        self.__collapsed = None
        self.__any_collapsed = False
        self.s_name = None
        self.adapter_removal_data = {}

        self.len_dist_plot_data = {
            'mate1': dict(),
            'mate2': dict(),
            'singleton': dict(),
            'collapsed': dict(),
            'collapsed_truncated': dict(),
            'discarded': dict(),
            'all': dict(),
        }

        parsed_data = None
        for f in self.find_log_files('adapterRemoval', filehandles=True):
            self.s_name = f['s_name']
            try:
                parsed_data = self.parse_settings_file(f)
            except UserWarning:
                continue
            if parsed_data is not None:
                self.adapter_removal_data[self.s_name] = parsed_data

        # Filter to strip out ignored sample names
        self.adapter_removal_data = self.ignore_samples(self.adapter_removal_data)

        if len(self.adapter_removal_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.adapter_removal_data)))

        # Write parsed report data to a file
        self.write_data_file(self.adapter_removal_data, 'multiqc_adapter_removal')

        # add data to Basic Stats Table
        self.adapter_removal_stats_table()

        self.adapter_removal_retained_chart()
        self.adapter_removal_length_dist_plot()

    def parse_settings_file(self, f):

        self.result_data = {
            'total': None,
            'unaligned': None,
            'aligned': None,
            'reads_total': None,
            'retained': None,
            'percent_aligned': None,
        }

        settings_data = {'header': []}

        block_title = None
        for i, line in enumerate(f['f']):

            line = line.rstrip('\n')
            if line == '':
                continue

            if not block_title:
                block_title = 'header'
                settings_data[block_title].append(str(line))
                continue

            if line.startswith('['):
                block_title = str(line.strip('[]'))
                settings_data[block_title] = []
                continue

            settings_data[block_title].append(str(line))

        # set data for further working
        self.set_result_data(settings_data)

        return self.result_data

    def set_result_data(self, settings_data):
        # set read and collapsed type
        self.set_ar_type(settings_data['Length distribution'])

        # set result_data
        self.set_trim_stat(settings_data['Trimming statistics'])
        self.set_len_dist(settings_data['Length distribution'])

    def set_ar_type(self, len_dist_data):
        head_line = len_dist_data[0].rstrip('\n').split('\t')

        self.__read_type = 'paired' if head_line[2] == 'Mate2' else 'single'
        if not self.__any_paired:
            self.__any_paired = True if head_line[2] == 'Mate2' else False

        self.__collapsed = True if head_line[-3] == 'CollapsedTruncated' else False
        if not self.__any_collapsed:
            self.__any_collapsed = True if head_line[-3] == 'CollapsedTruncated' else False

        # biological/technical relevance is not clear -> skip
        if self.__read_type == 'single' and self.__collapsed:
            log.warning("Case single-end and collapse is not " \
                        "implemented -> File %s skipped" % self.s_name)
            raise UserWarning

    def set_trim_stat(self, trim_data):
        required = ['total', 'unaligned', 'aligned', 'discarded_m1', 'singleton_m1', 'retained', 'discarded_m2', 'singleton_m2',
                    'full-length_cp', 'truncated_cp']
        data_pattern = {'total': 0,
                        'unaligned': 1,
                        'aligned': 2,
                        'discarded_m1': 3,
                        'singleton_m1': 4,
                        'retained': 6}

        if self.__read_type == 'paired':
            data_pattern['discarded_m2'] = 5
            data_pattern['singleton_m2'] = 6
            if not self.__collapsed:
                data_pattern['retained'] = 8
            else:
                data_pattern['full-length_cp'] = 8
                data_pattern['truncated_cp'] = 9
                data_pattern['retained'] = 10

        for field in required:
            if field in data_pattern:
                tmp = trim_data[data_pattern[field]]
                value = tmp.split(': ')[1]
                self.result_data[field] = int(value)
            else:
                self.result_data[field] = 0

        reads_total = self.result_data['total']
        aligned_total = self.result_data['aligned']
        unaligned_total = self.result_data['unaligned']
        if self.__read_type == 'paired':
            reads_total = self.result_data['total'] * 2
            aligned_total = self.result_data['aligned'] * 2
            unaligned_total = self.result_data['unaligned'] * 2

        self.result_data['aligned_total'] = aligned_total
        self.result_data['unaligned_total'] = unaligned_total
        self.result_data['reads_total'] = reads_total
        self.result_data['discarded_total'] = reads_total - self.result_data['retained']

        self.result_data['retained_reads'] = self.result_data['retained'] - self.result_data['singleton_m1'] - self.result_data['singleton_m2']
        try:
            self.result_data['percent_aligned'] = float(self.result_data['aligned']) * 100.0 / float(self.result_data['total'])
        except ZeroDivisionError:
            self.result_data['percent_aligned'] = 0

    def set_len_dist(self, len_dist_data):

        for line in len_dist_data[1:]:
            l_data = line.rstrip('\n').split('\t')
            l_data = list(map(int, l_data))

            # initialize file name
            if self.s_name not in self.len_dist_plot_data['mate1']:
                self.len_dist_plot_data['mate1'][self.s_name] = dict()
                self.len_dist_plot_data['mate2'][self.s_name] = dict()
                self.len_dist_plot_data['singleton'][self.s_name] = dict()
                self.len_dist_plot_data['collapsed'][self.s_name] = dict()
                self.len_dist_plot_data['collapsed_truncated'][self.s_name] = dict()
                self.len_dist_plot_data['discarded'][self.s_name] = dict()
                self.len_dist_plot_data['all'][self.s_name] = dict()

            if self.__read_type == 'single':
                if not self.__collapsed:
                    self.len_dist_plot_data['mate1'][self.s_name][l_data[0]] = l_data[1]
                    self.len_dist_plot_data['mate2'][self.s_name][l_data[0]] = None
                    self.len_dist_plot_data['singleton'][self.s_name][l_data[0]] = None
                    self.len_dist_plot_data['collapsed'][self.s_name][l_data[0]] = None
                    self.len_dist_plot_data['collapsed_truncated'][self.s_name][l_data[0]] = None
                    self.len_dist_plot_data['discarded'][self.s_name][l_data[0]] = l_data[2]
                    self.len_dist_plot_data['all'][self.s_name][l_data[0]] = l_data[3]
                else:
                    # this case should not be reached (see case at method set_ar_type())
                    pass
            else:
                if not self.__collapsed:
                    self.len_dist_plot_data['mate1'][self.s_name][l_data[0]] = l_data[1]
                    self.len_dist_plot_data['mate2'][self.s_name][l_data[0]] = l_data[2]
                    self.len_dist_plot_data['singleton'][self.s_name][l_data[0]] = l_data[3]
                    self.len_dist_plot_data['collapsed'][self.s_name][l_data[0]] = None
                    self.len_dist_plot_data['collapsed_truncated'][self.s_name][l_data[0]] = None
                    self.len_dist_plot_data['discarded'][self.s_name][l_data[0]] = l_data[4]
                    self.len_dist_plot_data['all'][self.s_name][l_data[0]] = l_data[5]
                else:
                    self.len_dist_plot_data['mate1'][self.s_name][l_data[0]] = l_data[1]
                    self.len_dist_plot_data['mate2'][self.s_name][l_data[0]] = l_data[2]
                    self.len_dist_plot_data['singleton'][self.s_name][l_data[0]] = l_data[3]
                    self.len_dist_plot_data['collapsed'][self.s_name][l_data[0]] = l_data[4]
                    self.len_dist_plot_data['collapsed_truncated'][self.s_name][l_data[0]] = l_data[5]
                    self.len_dist_plot_data['discarded'][self.s_name][l_data[0]] = l_data[6]
                    self.len_dist_plot_data['all'][self.s_name][l_data[0]] = l_data[7]

    def adapter_removal_stats_table(self):

        headers = OrderedDict()
        headers['percent_aligned'] = {
                'title': '% Trimmed',
                'description': '% trimmed reads',
                'max': 100,
                'min': 0,
                'suffix': '%',
                'scale': 'RdYlGn-rev',
                'shared_key': 'percent_aligned',
        }
        headers['aligned_total'] = {
                'title': '{} Reads Trimmed'.format(config.read_count_prefix),
                'description': 'Total trimmed reads ({})'.format(config.read_count_desc),
                'modify': lambda x: x * config.read_count_multiplier,
                'min': 0,
                'scale': 'PuBu',
                'shared_key': 'read_count'
        }
        self.general_stats_addcols(self.adapter_removal_data, headers)

    def adapter_removal_retained_chart(self):

        pconfig = {
            'title': 'Adapter Removal: Discarded Reads',
            'id': 'ar_retained_plot',
            'ylab': '# Reads',
            'hide_zero_cats': False,
            'cpswitch_counts_label': 'Number of Reads'
        }

        cats_pec = OrderedDict()

        if self.__any_paired:
            cats_pec['retained_reads'] = {'name': 'Retained Read Pairs'}

        cats_pec['singleton_m1'] = {'name': 'Singleton R1'}

        if self.__any_paired:
            cats_pec['singleton_m2'] = {'name': 'Singleton R2'}

            if self.__any_collapsed:
                cats_pec['full-length_cp'] = {'name': 'Full-length Collapsed Pairs'}
                cats_pec['truncated_cp'] = {'name': 'Truncated Collapsed Pairs'}

        cats_pec['discarded_m1'] = {'name': 'Discarded R1'}

        if self.__any_paired:
            cats_pec['discarded_m2'] = {'name': 'Discarded R2'}

        self.add_section(
            name='Retained and Discarded Paired-End Collapsed',
            anchor='adapter_removal_retained_plot',
            description='The number of retained and discarded reads.',
            plot=bargraph.plot(self.adapter_removal_data, cats_pec, pconfig)
        )

    def adapter_removal_length_dist_plot(self):

        pconfig = {
            'title': 'Adapter Removal: Length Distribution',
            'id': 'ar_length_count_plot',
            'ylab': 'Counts',
            'xlab': 'read length',
            'xDecimals': False,
            'ymin': 0,
            'tt_label': '<b>{point.x} bp trimmed</b>: {point.y:.0f}',
            'data_labels': None
        }

        lineplot_data = [
            self.len_dist_plot_data['all'],
            self.len_dist_plot_data['mate1']
        ]
        data_labels = [
            {'name': 'All', 'ylab': 'Count'},
            {'name': 'Mate1', 'ylab': 'Count'},
        ]
        if self.__any_paired:
            lineplot_data.extend([
                self.len_dist_plot_data['mate2'],
                self.len_dist_plot_data['singleton']
            ])
            data_labels.extend([
                {'name': 'Mate2', 'ylab': 'Count'},
                {'name': 'Singleton', 'ylab': 'Count'},
            ])
            if self.__any_collapsed:
                lineplot_data.extend([
                    self.len_dist_plot_data['collapsed'],
                    self.len_dist_plot_data['collapsed_truncated']
                ])
                data_labels.extend([
                    {'name': 'Collapsed', 'ylab': 'Count'},
                    {'name': 'Collapsed Truncated', 'ylab': 'Count'}
                ])
        lineplot_data.append(self.len_dist_plot_data['discarded'])
        data_labels.append({'name': 'Discarded', 'ylab': 'Count'})

        pconfig['data_labels'] = data_labels

        self.add_section(
            name='Length Distribution Paired End Collapsed',
            anchor='ar_length_count',
            description='The length distribution of reads after processing adapter alignment.',
            plot=linegraph.plot(lineplot_data, pconfig)
        )



#!/usr/bin/env python

""" MultiQC module to parse output from RSEM/rsem-calculate-expression """

from __future__ import print_function
import logging
from collections import OrderedDict

from multiqc import config
from multiqc.plots import bargraph, linegraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """
    RSEM module class, parses .cnt file .
    """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Rsem', anchor='rsem',
        href='https://deweylab.github.io/RSEM/',
        info="RSEM (RNA-Seq by Expectation-Maximization) is a software package for"\
             "estimating gene and isoform expression levels from RNA-Seq data.")

        self.rsem_mapped_data = dict()
        self.rsem_multimapping_data = dict()

        # Find and load any count file
        for f in self.find_log_files('rsem'):
            self.parse_rsem_report(f)

        # Filter to strip out ignored sample names
        self.rsem_mapped_data = self.ignore_samples(self.rsem_mapped_data)

        if len(self.rsem_mapped_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.rsem_mapped_data)))

        # Write parsed report data to a file
        self.write_data_file(self.rsem_mapped_data, 'multiqc_rsem')

        # Basic Stats Table
        self.rsem_stats_table()

        # Assignment bar plot
        self.rsem_mapped_reads_plot()

        # Multimapping line plot
        self.rsem_multimapping_plot()


    def parse_rsem_report (self, f):
        """ Parse the rsem cnt stat file.
        Description of cnt file found : https://github.com/deweylab/RSEM/blob/master/cnt_file_description.txt
        """
        data = dict()
        multimapping_hist = dict()
        in_hist = False
        for l in f['f'].splitlines():
            s = l.split()
            if len(s) > 3:
                # Line: N0 N1 N2 N_tot
                # N0, number of unalignable reads;
                # N1, number of alignable reads; (nUnique + nMulti)
                # N2, number of filtered reads due to too many alignments
                # N_tot = N0 + N1 + N2
                data['Unalignable'] = int(s[0])
                data['Alignable'] = int(s[1])
                data['Filtered'] = int(s[2])
                data['Total'] = int(s[3])
                data['alignable_percent'] = (float(s[1]) / float(s[3])) * 100.0
            elif len(s) == 3:
                # Line: nUnique nMulti nUncertain
                # nUnique, number of reads aligned uniquely to a gene
                # nMulti, number of reads aligned to multiple genes; nUnique + nMulti = N1;
                # nUncertain, number of reads aligned to multiple locations in the given reference sequences, which include isoform-level multi-mapping reads
                data['Unique'] = int(s[0])
                data['Multi'] = int(s[1])
                data['Uncertain'] = int(s[2])
            elif len(s) == 2:
                # Number of gene alignments and read count
                if in_hist or int(s[0]) == 0:
                    in_hist = True
                    try:
                        multimapping_hist[int(s[0])] = int(s[1])
                    except ValueError:
                        pass
            else:
                break
        try:
            assert data['Unique'] + data['Multi'] == data['Alignable']
        except AssertionError:
            log.warning("Unique + Multimapping read counts != alignable reads! '{}'".format(f['fn']))
            return None
        except KeyError:
            log.warning("Error parsing RSEM counts file '{}'".format(f['fn']))
            return None

        # Save parsed data
        if len(data) > 0:
            if f['s_name'] in self.rsem_mapped_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
            self.rsem_mapped_data[f['s_name']] = data
            self.add_data_source(f)
        if len(multimapping_hist) > 0:
            self.rsem_multimapping_data[f['s_name']] = multimapping_hist

    def rsem_stats_table(self):
        """ Take the parsed stats from the rsem report and add them to the
        basic stats table at the top of the report """
        headers = OrderedDict()
        headers['alignable_percent'] = {
            'title': '% Alignable'.format(config.read_count_prefix),
            'description': '% Alignable reads'.format(config.read_count_desc),
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'YlGn'
        }
        self.general_stats_addcols(self.rsem_mapped_data, headers)


    def rsem_mapped_reads_plot(self):
        """ Make the rsem assignment rates plot """

        # Plot categories
        keys = OrderedDict()
        keys['Unique'] =      { 'color': '#437bb1', 'name': 'Aligned uniquely to a gene' }
        keys['Multi'] =       { 'color': '#e63491', 'name': 'Aligned to multiple genes' }
        keys['Filtered'] =    { 'color': '#b1084c', 'name': 'Filtered due to too many alignments' }
        keys['Unalignable'] = { 'color': '#7f0000', 'name': 'Unalignable reads' }

        # Config for the plot
        config = {
            'id': 'rsem_assignment_plot',
            'title': 'RSEM: Mapped reads',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads',
            'hide_zero_cats': False
        }

        self.add_section(
            name = 'Mapped Reads',
            anchor = 'rsem_mapped_reads',
            description = 'A breakdown of how all reads were aligned for each sample.',
            plot = bargraph.plot(self.rsem_mapped_data, keys, config)
        )

    def rsem_multimapping_plot(self):
        """ Make a line plot showing the multimapping levels """

        pconfig = {
            'id': 'rsem_multimapping_rates',
            'title': 'RSEM: Multimapping Rates',
            'ylab': 'Counts',
            'xlab': 'Number of alignments',
            'xDecimals': False,
            'ymin': 0,
            'tt_label': '<b>{point.x} alignments</b>: {point.y:.0f}',
        }

        self.add_section(
            name = 'Multimapping rates',
            anchor = 'rsem_multimapping',
            description = 'A frequency histogram showing how many reads were aligned to `n` reference regions.',
            helptext = '''In an ideal world, every sequence reads would align uniquely to a single location in the
                reference. However, due to factors such as repeititve sequences, short reads and sequencing errors,
                reads can be align to the reference 0, 1 or more times. This plot shows the frequency of each factor
                of multimapping. Good samples should have the majority of reads aligning once.''',
            plot = linegraph.plot(self.rsem_multimapping_data, pconfig)
        )




#!/usr/bin/env python

""" MultiQC module to parse results from jellyfish  """

from __future__ import print_function

import logging
from multiqc.plots import linegraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name='Jellyfish',
            anchor='jellyfish',
            href="http://www.cbcb.umd.edu/software/jellyfish/",
            info="is a tool for fast, memory-efficient counting of k-mers in DNA."
        )

        self.jellyfish_data = dict()
        self.jellyfish_max_x = 0
        for f in self.find_log_files('jellyfish', filehandles=True):
            self.parse_jellyfish_data(f)

        if self.jellyfish_max_x < 100:
            # the maximum is below 100, we display anyway up to 200
            self.jellyfish_max_x = 200
        else:
            # in this case the area plotted is a function of the maximum x
            self.jellyfish_max_x = 2 * self.max_key

        self.jellyfish_data = self.ignore_samples(self.jellyfish_data)

        if len(self.jellyfish_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.jellyfish_data)))

        self.frequencies_plot(xmax=self.jellyfish_max_x)


    def parse_jellyfish_data(self, f):
        """ Go through the hist file and memorise it """
        histogram = {}
        occurence = 0
        for line in f['f']:
            line = line.rstrip('\n')
            occurence = int(line.split(" ")[0])
            count = int(line.split(" ")[1])
            histogram[occurence] = occurence*count
        #delete last occurnece as it is the sum of all kmer occuring more often than it.
        del histogram[occurence]
        #sanity check
        self.max_key  = max(histogram, key=histogram.get)
        self.jellyfish_max_x = max(self.jellyfish_max_x, self.max_key)
        if len(histogram) > 0:
            if f['s_name'] in self.jellyfish_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
            self.add_data_source(f)
            self.jellyfish_data[f['s_name']] = histogram


    def frequencies_plot(self, xmin=0, xmax=200):
        """ Generate the qualities plot """

        helptext = '''
            A possible way to assess the complexity of a library even in
            absence of a reference sequence is to look at the kmer profile of the reads.
            The idea is to count all the kmers (_i.e._, sequence of length `k`) that occur
            in the reads. In this way it is possible to know how many kmers occur
            `1,2,.., N` times and represent this as a plot.
            This plot tell us for each x, how many k-mers (y-axis) are present in the
            dataset in exactly x-copies.

            In an ideal world (no errors in sequencing, no bias, no  repeated regions)
            this plot should be as close as  possible to a gaussian distribution.
            In reality we will always see a peak for `x=1` (_i.e._, the errors)
            and another peak close to the expected coverage. If the genome is highly
            heterozygous a second peak at half of the coverage can be expected.'''

        pconfig = {
            'id': 'Jellyfish_kmer_plot',
            'title': 'Jellyfish: K-mer plot',
            'ylab': 'Counts',
            'xlab': 'k-mer frequency',
            'xDecimals': False,
            'xmin': xmin,
            'xmax': xmax
        }

        self.add_section(
            anchor = 'jellyfish_kmer_plot',
            description = 'The K-mer plot lets you estimate library complexity and coverage from k-mer content.',
            helptext = helptext,
            plot = linegraph.plot(self.jellyfish_data, pconfig)
        )



#!/usr/bin/env python

""" MultiQC module to parse output from SexdetErrmine """

from __future__ import print_function
from collections import OrderedDict
import logging
import json

from multiqc.plots import bargraph, scatter
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ SexDeterrmine module """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='SexDetErrmine', anchor='sexdeterrmine',
        href="https://github.com/TCLamnidis/Sex.DetERRmine",
        info="""A python script to calculate the relative coverage of X and Y chromosomes,
            and their associated error bars, from the depth of coverage at specified SNPs. """)

        # Find and load any DeDup reports
        self.sexdet_data = dict()

        # Find and load JSON file
        for f in self.find_log_files('sexdeterrmine', filehandles=True):
            self.parse_data(f)

        # Filter samples
        self.sexdet_data = self.ignore_samples(self.sexdet_data)

        # Return if no samples found
        if len(self.sexdet_data) == 0:
            raise UserWarning

        # Save data output file
        self.write_data_file(self.sexdet_data, 'multiqc_sexdeter_metrics')

        # Add to General Statistics
        self.addSummaryMetrics()

        # Plots
        self.snp_rate_scatterplot()
        self.read_count_barplot()
        self.snp_count_barplot()

    def parse_data(self, f):
        try:
            data = json.load(f['f'])
        except Exception as e:
            log.debug(e)
            log.warning("Could not parse SexDeterrmine JSON: '{}'".format(f['fn']))
            return

        # Parse JSON data to a dict
        for s_name in data:
            if (s_name == 'Metadata'):
                continue

            s_clean = self.clean_s_name(s_name, f['root'])
            if s_clean in self.sexdet_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(s_clean))

            self.add_data_source(f, s_clean)
            self.sexdet_data[s_clean] = dict()

            for k, v in data[s_name].items():
                try:
                    self.sexdet_data[s_clean][k] = float(v)
                except ValueError:
                    self.sexdet_data[s_clean][k] = v

    def addSummaryMetrics(self):
        """ Take the parsed stats from SexDetErrmine and add it to the main plot """

        headers = OrderedDict()
        headers['RateErrX'] = {
            'title': 'Err Rate X',
            'description': 'Rate of Error for Chr X',
            'scale': 'OrRd',
            'hidden': True,
            'shared_key': 'snp_err_rate'
        }
        headers['RateErrY'] = {
            'title': 'Err Rate Y',
            'description': 'Rate of Error for Chr Y',
            'scale': 'OrRd',
            'hidden': True,
            'shared_key': 'snp_err_rate'
        }
        headers['RateX'] = {
            'title': 'Rate X',
            'description': 'Number of positions on Chromosome X vs Autosomal positions.',
            'scale': 'PuBuGn',
            'shared_key': 'snp_count'
        }
        headers['RateY'] = {
            'title': 'Rate Y',
            'description': 'Number of positions on Chromosome Y vs Autosomal positions.',
            'scale': 'BuPu',
            'shared_key': 'snp_count'
        }

        self.general_stats_addcols(self.sexdet_data, headers)

    def read_count_barplot(self):
        """ Make a bar plot showing read counts on Autosomal, X and Y chr
        """
        cats = OrderedDict()
        cats['NR Aut'] = { 'name': 'Autosomal Reads' }
        cats['NrX'] = { 'name': 'Reads on X' }
        cats['NrY'] = { 'name': 'Reads on Y' }

        config = {
            'id': 'sexdeterrmine-readcounts-plot',
            'title': 'SexDetErrmine: Read Counts',
            'ylab': '# Reads'
        }

        self.add_section(
            name = 'Read Counts',
            anchor = 'sexdeterrmine-readcounts',
            description = 'The number of reads covering positions on the autosomes, X and Y chromosomes.',
            plot = bargraph.plot(self.sexdet_data, cats, config)
        )

    def snp_rate_scatterplot(self):
        """ Make a scatter plot showing relative coverage on X and Y chr
        """
        data = OrderedDict()
        for sample in self.sexdet_data:
            try:
                data[sample] = {'x': self.sexdet_data[sample]['RateX'], 'y': self.sexdet_data[sample]['RateY']}
            except KeyError:
                pass

        config = {
            'id': 'sexdeterrmine-rate-plot',
            'title': 'SexDetErrmine: Relative coverage',
            'ylab': 'Relative Cov. on Y',
            'xlab': 'Relative Cov. on X'
        }

        if len(data) > 0:
            self.add_section(
                name = 'Relative Coverage',
                anchor = 'sexdeterrmine-rates',
                description = 'The coverage on the X vs Y chromosome, relative to coverage on the Autosomes.',
                helptext = '''
                Males are expected to have a roughly equal X- and Y-rates, while females are expected to have a Y-rate of 0 and an X-rate of 1.
                Placement between the two clusters can be indicative of contamination, while placement with higher than expected X- and/or Y-rates can be indicative of sex chromosome aneuploidy.
                ''',
                plot = scatter.plot(data, config)
            )


    def snp_count_barplot(self):
        """ Make a bar plot showing read counts on Autosomal, X and Y chr
        """
        cats = OrderedDict()
        cats['Snps Autosomal'] = { 'name': 'Autosomal SNPs' }
        cats['XSnps'] = { 'name': 'SNPs on X' }
        cats['YSnps'] = { 'name': 'SNPs on Y' }

        config = {
            'id': 'sexdeterrmine-snps-plot',
            'title': 'SexDetErrmine: SNP Counts',
            'ylab': '# Reads'
        }

        self.add_section(
            name = 'SNP Counts',
            anchor = 'sexdeterrmine-snps',
            description = 'Total number of SNP positions. When supplied with a BED file, this includes only positions specified there.',
            plot = bargraph.plot(self.sexdet_data, cats, config)
        )



#!/usr/bin/env python

""" MultiQC module to parse output from pycoQC """

from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.plots import bargraph, table, linegraph
from multiqc import config
from collections import OrderedDict
import yaml
import logging

log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name = 'pycoQC',
            anchor = 'pycoqc',
            href = "https://a-slide.github.io/pycoQC/",
            info = "computes metrics and generates interactive QC plots for Oxford Nanopore technologies sequencing data"
        )

        self.pycoqc_data = {}
        for f in self.find_log_files('pycoqc'):
            if f['s_name'] in self.pycoqc_data:
                log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], f['s_name']))
            data = self.load_data(f['f'])
            # Function can return None if YAML parsing failed
            if data:
                self.pycoqc_data[f['s_name']] = data

        self.pycoqc_data = self.ignore_samples(self.pycoqc_data)

        # Stop if we didn't find anything
        if len(self.pycoqc_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.pycoqc_data)))

        self.parse_data()

        self.make_general_stats()
        self.make_pycoqc_table()
        self.read_base_count_plot()

        # Only add these sections if we have data (not available for older versions of PycoQC)
        if self.read_length_plot_data[0]:
            self.read_length_plot()

        if self.quality_plot_data[0]:
            self.make_quality_plot()

    def load_data(self, f):
        """ Load the PycoQC YAML file """
        try:
            return yaml.load(f, Loader=yaml.FullLoader)
        except Exception as e:
            log.warning("Could not parse YAML for '{}': \n  {}".format(f, e))
            return None

    def parse_data(self):
        """ Convert the parsed data into the correct structures for MultiQC functions """

        self.table_data = {}
        self.reads_data = {}
        self.bases_data = {}
        length_plot_all = {}
        length_plot_pass = {}
        qual_plot_all = {}
        qual_plot_pass = {}

        for sample, sample_data in self.pycoqc_data.items():
            self.table_data[sample] = {
                'all_median_read_length': sample_data['All Reads']['basecall']['len_percentiles'][50],
                'all_median_phred_score': sample_data['All Reads']['basecall']['qual_score_percentiles'][50],
                'all_n50': sample_data['All Reads']['basecall']['N50'],
                'all_run_duration': sample_data['All Reads']['run']['run_duration'],
                'all_channels': sample_data['All Reads']['run']['active_channels'],
                'all_reads': sample_data['All Reads']['basecall']['reads_number'],
                'all_bases': sample_data['All Reads']['basecall']['bases_number'],
                'passed_median_read_length': sample_data['Pass Reads']['basecall']['len_percentiles'][50],
                'passed_median_phred_score': sample_data['Pass Reads']['basecall']['qual_score_percentiles'][50],
                'passed_n50': sample_data['Pass Reads']['basecall']['N50'],
                'passed_channels': sample_data['Pass Reads']['run']['active_channels'],
                'passed_reads': sample_data['Pass Reads']['basecall']['reads_number'],
                'passed_bases': sample_data['Pass Reads']['basecall']['bases_number'],
                }

            self.reads_data[sample] = {
                'passed_reads': sample_data['Pass Reads']['basecall']['reads_number'],
                'non_passed_reads': sample_data['All Reads']['basecall']['reads_number'] - sample_data['Pass Reads']['basecall']['reads_number'],
            }

            self.bases_data[sample] = {
                'passed_bases': sample_data['Pass Reads']['basecall']['bases_number'],
                'non_passed_bases': sample_data['All Reads']['basecall']['bases_number'] - sample_data['Pass Reads']['basecall']['bases_number'],
            }
            try:
                length_x_vals_all = sample_data['All Reads']['basecall']['len_hist']['x']
                length_y_vals_all = sample_data['All Reads']['basecall']['len_hist']['y']
                length_plot_all[sample] = dict(zip(length_x_vals_all,length_y_vals_all))
                length_x_vals_pass = sample_data['Pass Reads']['basecall']['len_hist']['x']
                length_y_vals_pass = sample_data['Pass Reads']['basecall']['len_hist']['y']
                length_plot_pass[sample] = dict(zip(length_x_vals_pass,length_y_vals_pass))

                qual_x_vals_all = sample_data['All Reads']['basecall']['qual_score_hist']['x']
                qual_y_vals_all = sample_data['All Reads']['basecall']['qual_score_hist']['y']
                qual_plot_all[sample] = dict(zip(qual_x_vals_all,qual_y_vals_all))
                qual_x_vals_pass = sample_data['Pass Reads']['basecall']['qual_score_hist']['x']
                qual_y_vals_pass = sample_data['Pass Reads']['basecall']['qual_score_hist']['y']
                qual_plot_pass[sample] = dict(zip(qual_x_vals_pass,qual_y_vals_pass))
            except KeyError:
                log.debug("'{}': Could not find plot data. Please make sure you are using pycoQC v2.5.0.20 or newer.".format(sample))

        self.read_length_plot_data = [length_plot_pass, length_plot_all]
        self.quality_plot_data = [qual_plot_pass, qual_plot_all]

    def make_general_stats(self):
        general_stats_headers = OrderedDict()
        general_stats_headers['passed_median_read_length'] = {
            'title': 'Read Length - Pass (bp)',
            'description': 'Median Read Length - passing reads (base pairs)',
            'scale': 'BuPu',
            'shared_key': 'median_read_len',
            'format': '{:,.0f}',
        }
        general_stats_headers['all_median_read_length'] = {
            'title': 'Read Length - All (bp)',
            'description': 'Median Read Length - all reads (base pairs)',
            'scale': 'BuPu',
            'shared_key': 'median_read_len',
            'format': '{:,.0f}',
            'hidden': True
        }
        general_stats_headers['passed_reads'] = {
            'title': '{} Reads - Pass'.format(config.long_read_count_prefix),
            'description': 'Number of reads - passing reads ({})'.format(config.long_read_count_desc),
            'scale': 'BuGn',
            'modify': lambda x: x * config.long_read_count_multiplier,
            'shared_key': 'long_read_count',
        }
        general_stats_headers['all_reads'] = {
            'title': '{} Reads - All'.format(config.long_read_count_prefix),
            'description': 'Number of reads - all reads ({})'.format(config.long_read_count_desc),
            'scale': 'BuGn',
            'modify': lambda x: x * config.long_read_count_multiplier,
            'shared_key': 'long_read_count',
            'hidden': True
        }
        general_stats_headers['passed_bases'] = {
            'title': '{} Bases - Pass'.format(config.base_count_prefix),
            'description': 'Number of bases - passing reads ({} of base pairs)'.format(config.base_count_desc),
            'scale': 'OrRd',
            'modify': lambda x: x * config.base_count_multiplier,
            'shared_key': 'base_count',
        }
        general_stats_headers['all_bases'] = {
            'title': '{} Bases - All'.format(config.base_count_prefix),
            'description': 'Number of bases - all reads ({} of base pairs)'.format(config.base_count_desc),
            'scale': 'OrRd',
            'modify': lambda x: x * config.base_count_multiplier,
            'shared_key': 'base_count',
            'hidden': True
        }

        self.general_stats_addcols(self.table_data, general_stats_headers)

    def make_pycoqc_table(self):
        pycoqc_table_headers = OrderedDict()
        pycoqc_table_headers['passed_n50'] = {
            'title': 'N50 - Pass (bp)',
            'description': 'N50 - passing reads (base pairs)',
            'scale': 'BuGn',
            'shared_key': 'n50',
            'format': '{:,.0f}'
        }
        pycoqc_table_headers['all_n50'] = {
            'title': 'N50 - All (bp)',
            'description': 'N50 - all reads (base pairs)',
            'scale': 'BuGn',
            'shared_key': 'n50',
            'format': '{:,.0f}'
        }
        pycoqc_table_headers['passed_median_phred_score'] = {
            'title': 'Median read qual - Pass',
            'description': 'Median PHRED quality score - passing reads',
            'scale': 'PuRd',
            'shared_key': 'phred',
        }
        pycoqc_table_headers['all_median_phred_score'] = {
            'title': 'Median read qual - All',
            'description': 'Median PHRED quality score - all reads',
            'scale': 'PuRd',
            'shared_key': 'phred',
        }
        pycoqc_table_headers['passed_channels'] = {
            'title': 'Active Channels - Pass',
            'description': 'Number of active channels - passing reads',
            'scale': 'OrRd',
            'shared_key': 'channels',
            'format': '{:,.0f}'
        }
        pycoqc_table_headers['all_channels'] = {
            'title': 'Active Channels - All',
            'description': 'Number of active channels - all reads',
            'scale': 'OrRd',
            'shared_key': 'channels',
            'format': '{:,.0f}'
        }
        pycoqc_table_headers['all_run_duration'] = {
            'title': 'Run duration (h)',
            'description': 'Run duration (hours)',
            'scale': 'PuBuGn'
        }

        pycoqc_table_config = {
            'id': 'pycoqc_stats_table',
            'namespace': 'pycoQC'
        }

        self.add_section (
            name = 'Statistics',
            anchor = 'pycoqc_stats',
            plot = table.plot(self.table_data, pycoqc_table_headers, pycoqc_table_config)
        )

    def read_base_count_plot(self):
        """ Make a section with a bar plot showing the counts of reads and bases
        before and after filtering """

        # Plot config
        read_bar_config = {
            'id': 'pycoqc_count_plot',
            'title': 'pycoQC: Read Counts',
            'ylab': 'Sample',
            'data_labels': [
                {'name': 'Reads', 'ylab': 'Number of Reads'},
                {'name': 'Bases', 'ylab': 'Number of Base Pairs'}
            ]
        }

        # Two sets of categories for the two datasets
        read_cats = OrderedDict()
        read_cats['passed_reads'] = { 'name': 'Passed Reads' }
        read_cats['non_passed_reads'] = {'name': 'Failed Reads'}
        base_cats = OrderedDict()
        base_cats['passed_bases'] = { 'name': 'Passed Bases' }
        base_cats['non_passed_bases'] = {'name': 'Failed Bases'}

        # Make the plot
        bargraph_plot = bargraph.plot(
            [self.reads_data, self.bases_data],
            [read_cats, base_cats],
            read_bar_config
        )

        self.add_section(
            name = 'Read / Base counts',
            anchor = 'pycoqc_counts',
            description = 'Number of sequenced reads / bases passing and failing QC thresholds.',
            plot = bargraph_plot
        )

    def read_length_plot(self):
        """ Read length plot, showing passed reads and all reads """

        read_length_config = {
            'id': 'pycoqc_read_len_plot',
            'title': 'pycoQC: Read Length',
            'ylab': 'Read Density',
            'xlab': 'Basecalled Length (bp)',
            'xmin': 0,
            'data_labels': [
                {'name': 'Passing Reads', 'ylab': 'Read Density', 'xlab': 'Basecalled Length (bp)'},
                {'name': 'All Reads', 'ylab': 'Read Density', 'xlab': 'Basecalled Length (bp)'}
            ]
        }
        self.add_section(
            name = 'Read length',
            anchor = 'pycoqc_read_len',
            description = 'Distribution of read lengt for all / passed reads.',
            plot = linegraph.plot(
                self.read_length_plot_data,
                read_length_config
            )
        )

    def make_quality_plot(self):
        """ Quality plot, showing distrubtion of PHRED scores. """

        qual_config = {
            'id': 'pycoqc_read_qual_plot',
            'title': 'pycoQC: Read Quality Scores',
            'ylab': 'Read Density',
            'xlab': 'Read Quality PHRED Score',
            'xmin': 0,
            'data_labels': [
                {'name': 'Passing Reads', 'ylab': 'Read Density', 'xlab': 'Read Quality PHRED Score'},
                {'name': 'All Reads', 'ylab': 'Read Density', 'xlab': 'Read Quality PHRED Score'}
            ]
        }
        self.add_section(
            name = 'Quality scores',
            anchor = 'pycoqc_read_qual',
            description = 'Distribution of quality scores for all / passed reads.',
            plot = linegraph.plot(
                self.quality_plot_data,
                qual_config
            )
        )



#!/usr/bin/env python

""" MultiQC module to parse output from Kaiju """

from collections import OrderedDict
import logging

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name='Kaiju',
            anchor='kaiju',
            href="http://kaiju.binf.ku.dk/",
            info="a fast and sensitive taxonomic classification for metagenomics."
        )

        # Set up data structures
        self.kaiju_data = { }
        self.taxonomic_ranks = ["phylum","class","order","family","genus","species"]

        # Find and parse kaiju2table report
        for f in self.find_log_files('kaiju', filehandles=True):
            taxo_rank, parsed_data = self.parse_kaiju2table_report(f)
            if taxo_rank is not None and parsed_data is not None:
                # One report can have several samples, raise error if sample already viewed with this taxonomic rank
                s_names = parsed_data.keys()
                for s_name in s_names:
                    if taxo_rank in self.kaiju_data and s_name in self.kaiju_data[taxo_rank].keys() :
                        log.debug("Duplicate sample found in logs at {} rank! Overwriting sample: {}".format(taxo_rank, s_name))
                self.add_data_source(f)
                if taxo_rank in self.kaiju_data.keys() :
                    self.kaiju_data[taxo_rank].update(parsed_data)
                else :
                    self.kaiju_data[taxo_rank] = parsed_data

        # Filters to strip out ignored sample names
        for taxo_rank in self.kaiju_data :
            self.kaiju_data[taxo_rank] = self.ignore_samples(self.kaiju_data[taxo_rank])
            self.write_data_file(self.kaiju_data[taxo_rank], 'multiqc_kaiju_'+taxo_rank)

        # Number of samples found
        try:
            num_samples = max([len(taxo_rank) for taxo_rank in self.kaiju_data.values()])
        except ValueError:
            # No log files so didn't get any taxo_ranks
            raise UserWarning

        # no file found
        if num_samples == 0:
            raise UserWarning

        log.info("Found {} reports".format(num_samples))

        self.kaiju_stats_table()
        self.kaiju_taxo_rank_plot()

    def parse_kaiju2table_report(self, report):
        """ Search a kaiju with a set of regexes """
        parsed_data = {}
        s_names = []
        for l in report['f']:
            if l.startswith("file\t") :
                continue
            (s_file, pct, reads, taxon_id, taxon_names) = l.rstrip().split("\t")
            s_name = self.clean_s_name(s_file, report['root'])
            if s_name not in s_names :
                s_names.append(s_name)
                parsed_data[s_name]={"assigned":0, "unclassified":0, "cannot_be_assigned":0}
            if taxon_names.startswith("unclassified") :
                parsed_data[s_name]["unclassified"] = int(reads)
            elif taxon_names.startswith("cannot be assigned") :
                parsed_data[s_name]["cannot_be_assigned"] = int(reads)
                taxo_rank = taxon_names.split()[-1]
            else:
                parsed_data[s_name]["assigned"] += int(reads)

        for s_name in parsed_data :
            total_r = parsed_data[s_name]["assigned"] + parsed_data[s_name]["cannot_be_assigned"] + parsed_data[s_name]["unclassified"]
            parsed_data[s_name]['percentage_assigned'] = (float(parsed_data[s_name]["assigned"]) / float(total_r)) * 100.0

        return taxo_rank, parsed_data

    def kaiju_stats_table(self):
        """ Take the parsed stats from the Kaiju reports and add them to the
        basic stats table at the top of the report """
        headers = {}
        general_data ={}
        taxo_ranks = self.kaiju_data.keys()

        # print only phylum rank in general table.
        if len(taxo_ranks) >= 1 and "phylum" in taxo_ranks:
            general_data = self.kaiju_data["phylum"]
            general_taxo_rank = "Phylum"
        else:
            general_data = self.kaiju_data[self.kaiju_data.keys().sorted()[0]]
            general_taxo_rank = self.kaiju_data.keys().sorted()[0].capitalize()

        headers['percentage_assigned'] = {
            'title': '% Reads assigned {}'.format(general_taxo_rank),
            'description': 'Percentage of reads assigned at {} rank'.format(general_taxo_rank),
            'min': 0,
            'max': 100,
            'suffix': '%',
            'scale': 'RdYlGn'
        }
        headers['assigned'] = {
            'title': '{} Reads assigned {} '.format(config.read_count_prefix, general_taxo_rank),
            'description': 'Number of reads assigned ({})  at {} rank'.format(config.read_count_desc, general_taxo_rank),
            'modify': lambda x: x * config.read_count_multiplier,
            'scale': 'Blues'
        }
        self.general_stats_addcols(general_data, headers)

    def kaiju_taxo_rank_plot (self):
        """ Make the taxonomic rank barplot """

        pconfig = {
            'id': 'kaiju_taxo_rank_plot',
            'title': 'Kaiju: Reads assigned',
            'ylab': 'Number of reads',
            'cpswitch_counts_label': 'Number of reads',
            'data_labels':[]
        }
        datasets = []
        cats = []
        for rank in self.taxonomic_ranks :
            if rank in self.kaiju_data :
                pconfig['data_labels'].append( {'name': rank.capitalize(), 'ylab': 'Number of Reads at {} rank'.format(rank.capitalize()) })
                cats.append(OrderedDict ())
                cats[-1]['assigned'] = {'name': 'Assigned', 'color': '#1f78b4'}
                cats[-1]['cannot_be_assigned'] = {'name': 'Cannot be assigned at this rank', 'color': '#a6cee3'}
                cats[-1]['unclassified'] = {'name': 'Unclassified', 'color': '#ff7f00'}
                datasets.append(self.kaiju_data[rank])

        self.add_section (
            name = 'Read classification at different taxonomic levels',
            anchor = 'kaiju-classification',
            description = 'This plot shows the proportion of reads assigned, unclassified or not classified at different taxonomic rank.',
            plot = bargraph.plot(datasets, cats, pconfig)
        )



#!/usr/bin/env python

""" MultiQC module to parse output from Tophat """

from __future__ import print_function
from collections import OrderedDict
import logging
import os
import re

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Tophat', anchor='tophat',
        href="https://ccb.jhu.edu/software/tophat/",
        info="is a fast splice junction mapper for RNA-Seq reads. "\
        "It aligns RNA-Seq reads to mammalian-sized genomes.")

        # Find and load any Tophat reports
        self.tophat_data = dict()
        for f in self.find_log_files('tophat'):
            parsed_data = self.parse_tophat_log(f['f'])
            if parsed_data is not None:
                if (f['s_name'] == "align" or
                        f['s_name'] == 'align_summary.txt'):
                    s_name = os.path.basename(f['root'])
                else:
                    s_name = f['s_name'].split("align_summary.txt",1)[0]
                s_name = self.clean_s_name(s_name, f['root'])
                if s_name in self.tophat_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                self.add_data_source(f, s_name)
                self.tophat_data[s_name] = parsed_data

        # Filter to strip out ignored sample names
        self.tophat_data = self.ignore_samples(self.tophat_data)

        if len(self.tophat_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.tophat_data)))

        # Write parsed report data to a file
        self.write_data_file(self.tophat_data, 'multiqc_tophat.txt')

        # Basic Stats Table
        self.tophat_general_stats_table()

        # Alignment Rate Plot
        self.tophat_alignment_plot()


    def parse_tophat_log (self, raw_data):
        """ Parse the Tophat alignment log file. """

        if 'Aligned pairs' in raw_data:
            # Paired end data
            regexes = {
                'overall_aligned_percent': r"([\d\.]+)% overall read mapping rate.",
                'concordant_aligned_percent': r"([\d\.]+)% concordant pair alignment rate.",
                'aligned_total': r"Aligned pairs:\s+(\d+)",
                'aligned_multimap': r"Aligned pairs:\s+\d+\n\s+of these:\s+(\d+)",
                'aligned_discordant': r"(\d+) \([\s\d\.]+%\) are discordant alignments",
                'total_reads': r"[Rr]eads:\n\s+Input\s*:\s+(\d+)",
            }
        else:
            # Single end data
            regexes = {
                'total_reads': r"[Rr]eads:\n\s+Input\s*:\s+(\d+)",
                'aligned_total': r"Mapped\s*:\s+(\d+)",
                'aligned_multimap': r"of these\s*:\s+(\d+)",
                'overall_aligned_percent': r"([\d\.]+)% overall read mapping rate.",
            }

        parsed_data = {}
        for k, r in regexes.items():
            r_search = re.search(r, raw_data, re.MULTILINE)
            if r_search:
                parsed_data[k] = float(r_search.group(1))

        # Exit if we didn't manage to parse enough fields - probably not a TopHat log
        # Note that Bowtie2 / HiSAT2 logs contain some but not all of these strings
        if len(parsed_data) < 4: return None

        parsed_data['concordant_aligned_percent'] = parsed_data.get('concordant_aligned_percent', 0)
        parsed_data['aligned_total'] = parsed_data.get('aligned_total', 0)
        parsed_data['aligned_multimap'] = parsed_data.get('aligned_multimap', 0)
        parsed_data['aligned_discordant'] = parsed_data.get('aligned_discordant', 0)
        parsed_data['unaligned_total'] = parsed_data['total_reads'] - parsed_data['aligned_total']
        parsed_data['aligned_not_multimapped_discordant'] = parsed_data['aligned_total'] - parsed_data['aligned_multimap'] - parsed_data['aligned_discordant']
        return parsed_data


    def tophat_general_stats_table(self):
        """ Take the parsed stats from the Tophat report and add it to the
        basic stats table at the top of the report """

        headers = OrderedDict()
        headers['overall_aligned_percent'] = {
            'title': '% Aligned',
            'description': 'overall read mapping rate',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'YlGn'
        }
        headers['aligned_not_multimapped_discordant'] = {
            'title': '{} Aligned'.format(config.read_count_prefix),
            'description': 'Aligned reads, not multimapped or discordant ({})'.format(config.read_count_desc),
            'min': 0,
            'scale': 'PuRd',
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        self.general_stats_addcols(self.tophat_data, headers)

    def tophat_alignment_plot (self):
        """ Make the HighCharts HTML to plot the alignment rates """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['aligned_not_multimapped_discordant'] = { 'color': '#437bb1', 'name': 'Aligned' }
        keys['aligned_multimap'] =   { 'color': '#f7a35c', 'name': 'Multimapped' }
        keys['aligned_discordant'] = { 'color': '#e63491', 'name': 'Discordant mappings' }
        keys['unaligned_total'] =    { 'color': '#7f0000', 'name': 'Not aligned' }

        # Config for the plot
        config = {
            'id': 'tophat_alignment',
            'title': 'Tophat: Alignment Scores',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        self.add_section( plot =  bargraph.plot(self.tophat_data, keys, config) )
